SystemOrganization addCategory: #'Lisp-Interpreter'!
SystemOrganization addCategory: #'Lisp-Pluggable-Support'!

ListView subclass: #PluggableYAListView
	instanceVariableNames: 'items getListSelector getSelectionSelector setSelectionSelector getMenuSelector keystrokeActionSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lisp-Pluggable-Support'!

!PluggableYAListView commentStamp: 'nishis 6/28/1998 09:39' prior: 0!
from Squeak 1.31!

!PluggableYAListView class methodsFor: 'as yet unclassified' stamp: ''!
aboutPluggability
	"A pluggable list view gets its content from the model. This allows the same kind of view can be used in different situations, thus avoiding a proliferation of gratuitous view and controller classes. Selector usage is:

		getListSel		fetch the list of items (strings) to be displayed
		getSelectionSel	get the currently selected item
		setSelectionSel	set the currently selected item (takes an argument)
		getMenuSel		get the pane-specific (or 'yellow-button') menu
		keyActionSel	process keystrokes typed to this view (takes an argument)

	Any of the above selectors can be nil, meaning that the model does not supply behavior for the given action, and the default, do-nothing behavior should be used. However, if getListSel is nil, the default behavior just provides an empty list, which makes for a rather dull list view!! (Such behavior can actually be useful during debugging.)

	The model informs a pluggable view of changes by sending #changed: to itself with getListSel or getSelectionSel as a parameter. The view informs the model of selection changes by sending setSelectionSel to it with the newly selected item as a parameter, and invokes menu and keyboard actions on the model via getMenuSel and keyActionSel.

	Pluggability allows a single model object to have pluggable list views on multiple aspects of itself. For example, an object representing one personal music library might be organized as a three-level hierarchy: the types of music, the titles within a given type, and the songs on a given title. Pluggability allows one to easily build a multipane browser for this object with separate list views for the music type, title, and song."! !

!PluggableYAListView class methodsFor: 'as yet unclassified' stamp: ''!
on: anObject list: getListSel selected: getSelectionSel changeSelected: setSelectionSel
	"Create a 'pluggable' list view on the given model parameterized by the given message selectors. See aboutPluggability comment."

	^ self new
		on: anObject
		list: getListSel
		selected: getSelectionSel
		changeSelected: setSelectionSel
		menu: nil
		keystroke: nil
! !

!PluggableYAListView class methodsFor: 'as yet unclassified' stamp: ''!
on: anObject list: getListSel selected: getSelectionSel changeSelected: setSelectionSel menu: getMenuSel
	"Create a 'pluggable' list view on the given model parameterized by the given message selectors. See aboutPluggability comment."

	^ self new
		on: anObject
		list: getListSel
		selected: getSelectionSel
		changeSelected: setSelectionSel
		menu: getMenuSel
		keystroke: nil
! !

!PluggableYAListView class methodsFor: 'as yet unclassified' stamp: ''!
on: anObject list: getListSel selected: getSelectionSel changeSelected: setSelectionSel menu: getMenuSel keystroke: keyActionSel
	"Create a 'pluggable' list view on the given model parameterized by the given message selectors. See aboutPluggability comment."

	^ self new
		on: anObject
		list: getListSel
		selected: getSelectionSel
		changeSelected: setSelectionSel
		menu: getMenuSel
		keystroke: keyActionSel
! !

!PluggableYAListView methodsFor: 'model access' stamp: ''!
changeModelSelection: anInteger
	"Change the model's selected item to be the one at the given index."

	| item |
	setSelectionSelector ~~ nil ifTrue: [
		item _ (anInteger = 0 ifTrue: [nil] ifFalse: [items at: anInteger]).
		model perform: setSelectionSelector with: item.
		getSelectionSelector == nil ifFalse: [model perform: getSelectionSelector].
	].! !

!PluggableYAListView methodsFor: 'controller access' stamp: ''!
defaultControllerClass 

	^ PluggableYAListController! !

!PluggableYAListView methodsFor: 'model access' stamp: ''!
getCurrentSelectionIndex
	"Answer the index of the current selection."

	| item |
	getSelectionSelector == nil ifTrue: [^ 0].
	item _ model perform: getSelectionSelector.
	^ items findFirst: [ :x | x = item]
! !

!PluggableYAListView methodsFor: 'model access' stamp: ''!
getList 
	"Answer the list to be displayed."

	| lst |
	getListSelector == nil ifTrue: [^ #()].
	lst _ model perform: getListSelector.
	lst == nil ifTrue: [^ #()].
	^ lst! !

!PluggableYAListView methodsFor: 'model access' stamp: ''!
getMenu
	"Answer the menu for this list view."

	getMenuSelector == nil ifTrue: [^ nil].
	^ model perform: getMenuSelector! !

!PluggableYAListView methodsFor: 'model access' stamp: ''!
handleKeystroke: aCharacter
	"Answer the menu for this list view."

	keystrokeActionSelector == nil ifTrue: [^ nil].
	model perform: keystrokeActionSelector with: aCharacter.! !

!PluggableYAListView methodsFor: 'initialization' stamp: ''!
list: arrayOfStrings
	"Set the receivers items to be the given list of strings."
	"Note: the instance variable 'itemsList' holds the original list.
	 The instance variable 'items' is a paragraph constructed from
	 this list."

	| s |
	items _ arrayOfStrings.
	isEmpty _ arrayOfStrings isEmpty.

	s _ WriteStream on: Array new.
	"add top and bottom delimiters"
	s nextPut: topDelimiter.
	arrayOfStrings do: [:item |
		item == nil ifFalse: [s nextPut: item].
	].
	s nextPut: bottomDelimiter.
	list _ ListParagraph withArray: s contents.

	selection _ self getCurrentSelectionIndex.
	self positionList.! !

!PluggableYAListView methodsFor: 'initialization' stamp: ''!
on: anObject list: getListSel selected: getSelectionSel changeSelected: setSelectionSel menu: getMenuSel keystroke: keyActionSel

	self model: anObject.
	getListSelector _ getListSel.
	getSelectionSelector _ getSelectionSel.
	setSelectionSelector _ setSelectionSel.
	getMenuSelector _ getMenuSel.
	keystrokeActionSelector _ keyActionSel.
	self borderWidth: 1.
	self list: self getList.! !

!PluggableYAListView methodsFor: 'updating' stamp: ''!
update: aSymbol 
	"Refer to the comment in View|update:."

	| oldIndex newIndex |
	aSymbol == getListSelector ifTrue: [
		oldIndex _ self getCurrentSelectionIndex.
		self list: self getList.
		newIndex _ self getCurrentSelectionIndex.
		(oldIndex > 0 and: [newIndex = 0]) ifTrue: [
			"new list did not include the old selection; deselecting"
			self changeModelSelection: newIndex].
		self displayView.
		self displaySelectionBox.
		^self].
	aSymbol == getSelectionSelector ifTrue: [
		self moveSelectionBox: self getCurrentSelectionIndex.
		^self].
! !

StringHolder subclass: #PluggableStringHolderModel
	instanceVariableNames: 'selectionMenu'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lisp-Pluggable-Support'!

!PluggableStringHolderModel commentStamp: 'nishis 6/28/1998 09:39' prior: 0!
#
# This file is encoded in (so called) SHIFT-JIS,
# line delimitor is CR.
#
PluggableStringHolderModel (Pluggable-StringHolderMVC)
Goodies for Squeak 1.31
This goodies for Squeak from Mr. AOKI
!

PluggableStringHolderModel subclass: #LispInterpreter
	instanceVariableNames: 'lispTable bindStack failBlock textValue textCollector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lisp-Interpreter'!

!LispInterpreter commentStamp: 'nishis 6/28/1998 09:39' prior: 0!
#
# This file is encoded in (so called) SHIFT-JIS,
# line delimitor is CR.
#
Lisp
Goodies for Squeak 2.0

Broken English description

This goodies is a modification for Squeak from Mr. AOKI's
Lisp
Goodies for VisualWorks 2.5 and VisualWave 1.0
Copyright (c) 1995-1998 AOKI Atsushi
1998/01/05
http://www.sra.co.jp/people/aoki/SmalltalkGoodies/VisualWorks/Lisp.st
http://www.sra.co.jp/people/aoki/SmalltalkGoodies/VisualWorks/Lisp.txt
!

!LispInterpreter class methodsFor: 'copyright' stamp: ''!
copyright
	^'Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.'! !

!LispInterpreter class methodsFor: 'error handling' stamp: 'nishis 6/28/1998 06:04'!
error: labelString with: contentsString
	| handler |
	(handler _ Processor activeProcess errorHandler) notNil
		ifTrue:
			[handler value: labelString value: self]
		ifFalse: 
			[Debugger
				openContext: thisContext
				label: labelString
				contents: contentsString]! !

!LispInterpreter class methodsFor: 'public access' stamp: 'nishis 4/25/98 04:07'!
evaluate: sExpression 
	^self new evaluateTopLevel: sExpression ifFail: [:errorMessage | self error: 'cannot evaluate:' with:  errorMessage]! !

!LispInterpreter class methodsFor: 'public access' stamp: ''!
evaluate: sExpression ifFail: aBlock 
	^self new evaluateTopLevel: sExpression ifFail: aBlock! !

!LispInterpreter class methodsFor: 'public access' stamp: 'nishis 4/25/98 04:07'!
evaluateFrom: aStream 
	^self new evaluateTopLevel: (LispParser parse: aStream)
		ifFail: [:errorMessage | self error: 'cannot evaluate from:' with: errorMessage]! !

!LispInterpreter class methodsFor: 'public access' stamp: ''!
evaluateFrom: aStream ifFail: aBlock 
	^self new evaluateTopLevel: (LispCons parse: aStream ifFail: aBlock)
		ifFail: aBlock! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example
	"LispInterpreter example."

	LispInterpreter open! !

!LispInterpreter class methodsFor: 'examples' stamp: 'nishis 5/3/98 01:56'!
example00
	"LispInterpreter example00."

	LispInterpreter fullOpen! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example01
	"LispInterpreter example01."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	nil

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example02
	"LispInterpreter example02."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	123

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example03
	"LispInterpreter example03."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	"abc"

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example04
	"LispInterpreter example04."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	(cons 3 4)

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example05
	"LispInterpreter example05."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	(quote (3 4))

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example06
	"LispInterpreter example06."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	(car (quote (3 4)))

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example07
	"LispInterpreter example07."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	(cdr (quote (3 4)))

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example08
	"LispInterpreter example08."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	(cons (car `(1 2 3)) `(3 4))

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example09
	"LispInterpreter example09."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	(+ 1 2 3 4 5 6 7 8 9 10)

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example10
	"LispInterpreter example10."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	(progn
		(setq x 100)
		(setq y 200 z 300)
		(+ x y z))

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example11
	"LispInterpreter example11."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	(progn
		(defun plus (x y) (+ x y))
		(plus 3 4))

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example12
	"LispInterpreter example12."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	(progn
		(defun concat (x y) 
			(cond
				((atom x) y)
				(t (cons
						(car x)
						(concat (cdr x) y)))))
		(concat `(1 2 3) `(4 5)))

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example13
	"LispInterpreter example13."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	(do
		(image display point)
		(setq image {Form fromUser})
		(setq display {Display})
		(setq point {50@50})
		(send image `displayOn:at: display point))

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example14
	"LispInterpreter example14."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	(progn
		(defun plus nlambda (x) (plus1 x))
		(defun plus1 lambda (x)
			(cond
				((null x) 0)
				(t (+ (car x) (plus1 (cdr x))))))
		(plus 1 2 3 4 5 6 7 8 9 10))

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example15
	"LispInterpreter example15."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	((lambda (x y) (cons x (cons y nil))) 3 4)

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example16
	"LispInterpreter example16."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	((nlambda (x) x) 1 2 3 4 5 6 7 8 9 10)

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example17
	"LispInterpreter example17."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	(progn
		(setq x 100)
		(setq y 200)
		(do
			(x y)
			(setq x 10000)
			(setq y 20000)
			(send {Transcript} `cr)
			(send {Transcript} `show: (send x `printString))
			(send {Transcript} `cr)
			(send {Transcript} `show: (send y `printString)))
		(send {Transcript} `cr)
		(send {Transcript} `show: (send x `printString))
		(send {Transcript} `cr)
		(send {Transcript} `show: (send y `printString)))

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example18
	"LispInterpreter example18."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	(do
		(count)
		(setq count 1)
		(while
			(<= count 100)
			do
			(send {Transcript} `cr)
			(send {Transcript} `show: (send count `printString))
			(setq count (+ count 1))))

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example19
	"LispInterpreter example18."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	(do
		(count)
		(setq count 1)
		(repeat
			(send {Transcript} `cr)
			(send {Transcript} `show: (send count `printString))
			(setq count (+ count 1))
			until
			(<= count 100)))

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: ''!
example20
	"LispInterpreter example20."

	| aList |
	aList := LispInterpreter evaluateFrom: '

	(progn
		(if
			(= (send {self} `confirm: "Are you happy?") {true})
			then
			(send {Transcript} `cr)
			(send {Transcript} `show: "yes")
			else 
			(send {Transcript} `cr)
			(send {Transcript} `show: "no")))

	'.
	^aList! !

!LispInterpreter class methodsFor: 'examples' stamp: 'nishis 5/3/98 13:38'!
example21
	"LispInterpreter example21."

	| anInterpreter aBlock aString |
	anInterpreter := LispInterpreter new.
	aString := '	(progn
		(defun concat (x y) 
			(cond
				((atom x) y)
				(t (cons
						(car x)
						(concat (cdr x) y)))))
		(concat `(1 2 3) `(4 5)))
'.
	anInterpreter contents: aString.
	aBlock :=
		['> (progn
		(defun concat (x y) 
			(cond
				((atom x) y)
				(t (cons
						(car x)
						(concat (cdr x) y)))))
		(concat `(1 2 3) `(4 5)))
(1 2 3 4 5)'
			do:
				[:aCharacter |
				Processor yield.
				aString := String with: aCharacter.
				anInterpreter textCollector show: aString]].
	aBlock fixTemps; fork.
	anInterpreter open! !

!LispInterpreter class methodsFor: 'instance creation' stamp: 'nishis 5/3/98 01:56'!
fullOpen
	"LispInterpreter fullOpen."

	self fullOpenOn: self new! !

!LispInterpreter class methodsFor: 'instance creation' stamp: 'nishis 5/3/98 01:56'!
fullOpenOn: lispInterpreter
	"LispInterpreter fullOpen."

	lispInterpreter fullOpen! !

!LispInterpreter class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize! !

!LispInterpreter class methodsFor: 'instance creation' stamp: 'nishis 4/21/98 00:57'!
open
	"LispInterpreter open."

	self openOn: self new! !

!LispInterpreter class methodsFor: 'instance creation' stamp: 'nishis 4/21/98 00:57'!
openOn: lispInterpreter
	"LispInterpreter open."

	lispInterpreter open! !

!LispInterpreter class methodsFor: 'printing' stamp: ''!
printString: anObject 
	"LispInterpreter printString: 'string'."
	"LispInterpreter printString: #symbol."
	"LispInterpreter printString: 123."
	"LispInterpreter printString: 123.456."
	"LispInterpreter printString: 123.456e7."
	"LispInterpreter printString: LispNil null."
	"LispInterpreter printString: nil."

	| aStream |
	aStream := WriteStream on: (String new: 32).
	(anObject isKindOf: LispCons)
		ifTrue: [anObject printOn: aStream level: 0]
		ifFalse: [LispCons new printOn: aStream object: anObject].
	^aStream contents! !

!LispInterpreter class methodsFor: 'saving' stamp: 'nishis 6/28/1998 06:59'!
save
	"LispInterpreter save."

	| fileName classCollection aStream timeStamp |
	fileName := 'Lisp.st'.
	classCollection := self saveClasses.
	aStream := FileStream newFileNamed: fileName.
	aStream reset.
	Cursor write
		showWhile: 
			[timeStamp := Date today mmddyyyy, ' ' , Time now asString.
			aStream cr.
			aStream nextChunkPut: timeStamp printString.
			aStream cr; cr.
			(self comment isNil or: [self comment isEmpty])
				ifFalse: 
					[aStream nextChunkPut: (self comment asString) printString.
					aStream cr; cr].
			classCollection
				do: 
					[:aClass | 
					aStream nextChunkPut: aClass definition.
					aStream cr; cr].
			classCollection
				do: 
					[:aClass | 
					aStream nextPut: Character newPage.
					aStream cr.
					aClass fileOutOn: aStream.
					aStream cr]].
	aStream close.
	^classCollection! !

!LispInterpreter class methodsFor: 'saving' stamp: 'nishis 6/13/1998 03:49'!
saveClasses
	"LispInterpreter saveClasses."

	| patternCollection classCollection string something |
	patternCollection := #('Lisp*' 'Pluggable*').
	classCollection := SystemOrganization superclassOrder: self category.
	classCollection addAll: (SystemOrganization superclassOrder: 'Pluggable-Support').
	classCollection := ChangeSet superclassOrder: classCollection.
	classCollection := classCollection
				select: 
					[:aClass | 
					string := aClass name asString.
					something := patternCollection detect: [:it | it match: string]
								ifNone: [nil].
					something notNil].
	^classCollection! !

!LispInterpreter class methodsFor: 'copyright' stamp: ''!
system
	^'Goodies'! !

!LispInterpreter class methodsFor: 'copyright' stamp: ''!
version
	^'003'! !

!LispInterpreter methodsFor: 'private' stamp: 'nishis 6/12/1998 02:43'!
acceptWith: aText
	"force to regist aText as a contents."

	self contents: aText.
	textValue := aText.
	self changed: #clearUserEdits! !

!LispInterpreter methodsFor: 'shallow binding' stamp: ''!
bind: symbol value: value 
	| saveValue assoc |
	lispTable intern: symbol.
	saveValue := lispTable getprop: symbol key: #apval.
	assoc := Association key: symbol value: saveValue.
	bindStack addLast: assoc.
	self
		putprop: symbol
		key: #apval
		value: value! !

!LispInterpreter methodsFor: 'shallow binding' stamp: ''!
bindMark
	| assoc |
	assoc := Association key: nil value: nil.
	bindStack addLast: assoc! !

!LispInterpreter methodsFor: 'private' stamp: 'nishis 6/12/1998 02:18'!
buttonView: baseView onAction: aAction label: label extent: extent
	| aSwitchView aSwitch |
	aSwitch := (Button new) onAction: aAction.
	aSwitchView := PluggableButtonView
		on: aSwitch
		getState: #isOff
		action: #switch.
	aSwitchView
		label: label asParagraph;
		window: (0@0 extent:  extent);
		borderWidth: 2;
			foregroundColor: Color black;
			backgroundColor: Color veryVeryLightGray.
	^aSwitchView! !

!LispInterpreter methodsFor: 'private' stamp: 'nishis 4/27/98 04:34'!
buttonView: baseView onAction: aAction label: label extent: extent align: align targetView: targetView
	| aSwitchView kind |
	aSwitchView := 
		self 
			buttonView: baseView 
			onAction: aAction 
			label: label 
			extent: extent.
	kind := align numArgs.
	kind = 1 ifTrue: [baseView perform: align asSymbol with: aSwitchView].
	kind = 2 ifTrue: [baseView perform: align asSymbol with: aSwitchView with: targetView].
	^aSwitchView
! !

!LispInterpreter methodsFor: 'interface opening' stamp: 'nishis 4/27/98 04:36'!
buttonsView: aController
	| baseView doItView inspectItView gcView width height |
	width := 250 / 4.
	height := 16.
	baseView := View new 
			model: self;
			backgroundColor: Color white;
			window: (0 @ 0 extent: 250 @ height);
			borderWidthLeft: 2 right: 2 top: 2 bottom: 0.
	doItView := self 
		buttonView: baseView 
		onAction: [self doIt: aController selection from: aController] 
		label: 'do it' 
		extent: width @ height 
		align: #addSubView:
		targetView: nil.
	inspectItView := self 
		buttonView: baseView 
		onAction: [self inspectIt: aController selection from: aController] 
		label: 'inspect it' 
		extent: width @ height 
		align: #addSubView:toRightOf: 
		targetView: doItView.
	gcView := self 
		buttonView: baseView 
		onAction: [self perform: #subrGc] 
		label: 'gc' 
		extent: width @ height 
		align: #addSubView:toRightOf: 
		targetView: inspectItView.
	self 
		buttonView: baseView 
		onAction: [self perform: #investigate] 
		label: 'investigate' 
		extent: width @ height 
		align: #addSubView:toRightOf: 
		targetView: gcView.
	^baseView
! !

!LispInterpreter methodsFor: 'menu messages' stamp: 'nishis 5/3/98 10:46'!
doIt: aText from: aController 
	| stream result |
	self acceptWith: aText.
	"see BrowserView browseFullForClass: PluggableStringHolderController method: #sendMenuMessageToModel:."
	stream := ReadStream on: aText asString.
	result := LispNil null.
	[stream atEnd]
		whileFalse: 
			[result := self execute: stream.
			self skipSeparators: stream].
	^result! !

!LispInterpreter methodsFor: 'error handling' stamp: 'nishis 6/28/1998 06:04'!
error: labelString with: contentsString
	self class error: labelString with: contentsString! !

!LispInterpreter methodsFor: 'evaluating' stamp: ''!
evaluate: sExpression 
	| apval |
	(sExpression isKindOf: LispList)
		ifTrue: [^self listEvaluate: sExpression].
	(sExpression isKindOf: Symbol)
		ifTrue: 
			[sExpression = #t ifTrue: [^#t].
			lispTable intern: sExpression.
			apval := lispTable getprop: sExpression key: #apval.
			apval isNil ifTrue: [^self fatal: (self printString: sExpression)
						, ' is unbound atom'].
			^apval].
	^sExpression! !

!LispInterpreter methodsFor: 'evaluating' stamp: ''!
evaluateTopLevel: sExpression 
	| result |
	failBlock := 
			[:errorMessage | 
			textCollector show: errorMessage; cr.
			Cursor normal show.
			^LispNil null].
	Cursor execute showWhile: [result := self evaluate: sExpression].
	^result! !

!LispInterpreter methodsFor: 'evaluating' stamp: ''!
evaluateTopLevel: sExpression ifFail: aBlock 
	failBlock := aBlock.
	^self evaluate: sExpression! !

!LispInterpreter methodsFor: 'executing' stamp: ''!
execute: aStream 
	| start list end string result |
	self skipSeparators: aStream.
	start := aStream position + 1.
	list := LispParser parse: aStream.
	end := aStream position.
	string := aStream contents copyFrom: start to: end.
	textCollector show: '> ' , string.
	textCollector cr.
	result := self evaluateTopLevel: list.
	textCollector show: (self printString: result).
	textCollector cr.
	^result! !

!LispInterpreter methodsFor: 'func eval' stamp: ''!
exprEval: funcBody arguList: arguList 
	| expression funcName lvarList lvar result |
	expression := funcBody.
	funcName := expression head.
	expression := expression tail.
	expression := expression tail.
	lvarList := expression head.
	expression := expression tail.
	arguList length ~= lvarList length ifTrue: [^self fatal: 'too few or many arguments ' , (self printString: arguList) , ' vs ' , (self printString: lvarList) , ' for ' , funcName].
	self bindMark.
	arguList
		do: 
			[:each | 
			lvar := lvarList head.
			self bind: lvar value: (self evaluate: each).
			lvarList := lvarList tail].
	expression do: [:each | result := self evaluate: each].
	self unbind.
	^result! !

!LispInterpreter methodsFor: 'error handling' stamp: ''!
fatal: message 
	bindStack reverseDo: [:assoc | assoc key notNil ifTrue: [assoc value notNil
				ifTrue: [self
						putprop: assoc key
						key: #apval
						value: assoc value]
				ifFalse: [self remprop: assoc key key: #apval]]].
	^failBlock value: '*** Error: ' , message! !

!LispInterpreter methodsFor: 'func eval' stamp: ''!
fexprEval: funcBody arguList: arguList 
	| expression funcName lvarList lvar result |
	expression := funcBody.
	funcName := expression head.
	expression := expression tail.
	expression := expression tail.
	lvarList := expression head.
	expression := expression tail.
	lvarList length ~= 1 ifTrue: [^self fatal: 'too few or many arguments ' , (self printString: lvarList) , ' for ' , (self printString: funcName)].
	self bindMark.
	lvar := lvarList head.
	self bind: lvar value: arguList.
	expression do: [:each | result := self evaluate: each].
	self unbind.
	^result! !

!LispInterpreter methodsFor: 'fsubr functions' stamp: ''!
fsubrAdd: arguList 
	| v a |
	v := LispNil null.
	arguList
		do: 
			[:each | 
			a := self evaluate: each.
			(a isKindOf: Number)
				ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a) , ' for +'].
			v = LispNil null
				ifTrue: [v := a]
				ifFalse: [v := v + a]].
	^v! !

!LispInterpreter methodsFor: 'fsubr functions' stamp: ''!
fsubrCond: arguList 
	| result |
	arguList do: [:each | (self evaluate: each head)
			~= LispNil null
			ifTrue: 
				[result := LispNil null.
				(each tail isKindOf: LispCons)
					ifTrue: [each tail do: [:expr | result := self evaluate: expr]].
				^result]].
	^LispNil null! !

!LispInterpreter methodsFor: 'fsubr functions' stamp: ''!
fsubrDefun: arguList 
	| funcName funcType |
	funcName := arguList head.
	(funcName isKindOf: Symbol)
		ifFalse: [^self fatal: 'unexpected function name ' , (self printString: funcName) , ' for defun'].
	funcType := arguList tail head.
	funcType = #lambda
		ifTrue: 
			[self
				putprop: funcName
				key: #expr
				value: arguList.
			^funcName].
	funcType = #nlambda
		ifTrue: 
			[self
				putprop: funcName
				key: #fexpr
				value: arguList.
			^funcName].
	self
		putprop: funcName
		key: #expr
		value: (LispCons head: funcName tail: (LispCons head: #lambda tail: arguList tail)).
	^funcName! !

!LispInterpreter methodsFor: 'fsubr functions' stamp: ''!
fsubrDiv: arguList 
	| v a |
	v := LispNil null.
	arguList
		do: 
			[:each | 
			a := self evaluate: each.
			(a isKindOf: Number)
				ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a) , ' for /'].
			v = LispNil null
				ifTrue: [v := a]
				ifFalse: [v := v / a]].
	^v! !

!LispInterpreter methodsFor: 'fsubr functions' stamp: ''!
fsubrDo: arguList 
	| locals executions result |
	locals := arguList head.
	executions := arguList tail.
	self bindMark.
	locals do: [:lvar | self bind: lvar value: LispNil null].
	result := LispNil null.
	executions do: [:each | result := self evaluate: each].
	self unbind.
	^result! !

!LispInterpreter methodsFor: 'func eval' stamp: ''!
fsubrEval: funcBody arguList: arguList 
	| messageSelector |
	messageSelector := funcBody tail head.
	^self perform: messageSelector with: arguList! !

!LispInterpreter methodsFor: 'fsubr functions' stamp: ''!
fsubrIdiv: arguList 
	| v a |
	v := LispNil null.
	arguList
		do: 
			[:each | 
			a := self evaluate: each.
			(a isKindOf: Number)
				ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a) , ' for //'].
			v = LispNil null
				ifTrue: [v := a]
				ifFalse: [v := v // a]].
	^v! !

!LispInterpreter methodsFor: 'fsubr functions' stamp: ''!
fsubrIf: arguList 
	| predicate then list truePart falsePart bool result |
	predicate := arguList head.
	then := arguList tail head.
	list := arguList tail tail.
	truePart := LispNil null.
	falsePart := LispNil null.
	bool := true.
	list do: [:each | each = #else
			ifTrue: [bool := false]
			ifFalse: [bool
					ifTrue: [truePart := LispCons head: each tail: truePart]
					ifFalse: [falsePart := LispCons head: each tail: falsePart]]].
	then = #then ifFalse: [^self fatal: 'unexpected format for if'].
	truePart := truePart reverse.
	falsePart := falsePart reverse.
	result := LispNil null.
	(self evaluate: predicate)
		= LispNil null
		ifTrue: [falsePart do: [:each | result := self evaluate: each]]
		ifFalse: [truePart do: [:each | result := self evaluate: each]].
	^result! !

!LispInterpreter methodsFor: 'fsubr functions' stamp: ''!
fsubrMlt: arguList 
	| v a |
	v := LispNil null.
	arguList
		do: 
			[:each | 
			a := self evaluate: each.
			(a isKindOf: Number)
				ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a) , ' for *'].
			v = LispNil null
				ifTrue: [v := a]
				ifFalse: [v := v * a]].
	^v! !

!LispInterpreter methodsFor: 'fsubr functions' stamp: ''!
fsubrMod: arguList 
	| v a |
	v := LispNil null.
	arguList
		do: 
			[:each | 
			a := self evaluate: each.
			(a isKindOf: Number)
				ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a) , ' for \\'].
			v = LispNil null
				ifTrue: [v := a]
				ifFalse: [v := v \\ a]].
	^v! !

!LispInterpreter methodsFor: 'fsubr functions' stamp: ''!
fsubrProgn: arguList 
	| result |
	result := LispNil null.
	arguList do: [:each | result := self evaluate: each].
	^result! !

!LispInterpreter methodsFor: 'fsubr functions' stamp: ''!
fsubrQuote: arguList
	^arguList head! !

!LispInterpreter methodsFor: 'fsubr functions' stamp: ''!
fsubrRead: arguList 
	| message default answer theClipboardText |
	message := '          Please type S expression.          '.
	default := String new.
	arguList length >= 1
		ifTrue: 
			[message := self evaluate: (arguList nth: 1).
			(message isKindOf: String)
				ifFalse: [message := self printString: message]].
	arguList length >= 2
		ifTrue: 
			[default := self evaluate: (arguList nth: 2).
			(default isKindOf: String)
				ifFalse: [default := self printString: default]].
	theClipboardText := Smalltalk clipboardText.
	Smalltalk clipboardText: ''.
	answer := FillInTheBlank request: message initialAnswer: default.
	Smalltalk clipboardText: theClipboardText.
	(answer isNil or: [answer isEmpty])
		ifTrue: [^LispNil null].
	^LispParser parse: (ReadStream on: answer)! !

!LispInterpreter methodsFor: 'fsubr functions' stamp: ''!
fsubrRepeat: arguList 
	| reverse predicate until executions result |
	reverse := arguList reverse.
	predicate := reverse head.
	until := reverse tail head.
	executions := reverse tail tail reverse.
	until = #until ifFalse: [^self fatal: 'unexpected format for repeat'].
	result := LispNil null.
	executions do: [:each | result := self evaluate: each].
	[(self evaluate: predicate)
		= LispNil null]
		whileFalse: [executions do: [:each | result := self evaluate: each]].
	^result! !

!LispInterpreter methodsFor: 'fsubr functions' stamp: ''!
fsubrSend: arguList 
	| list receiver selector arguments result |
	list := arguList.
	receiver := self evaluate: list head.
	list := list tail.
	selector := self evaluate: list head.
	(selector isKindOf: Symbol)
		ifFalse: [^self fatal: 'unexpected selector ' , (self printString: selector) , ' for send'].
	list := list tail.
	arguments := OrderedCollection new.
	[list isKindOf: LispCons]
		whileTrue: 
			[arguments add: (self evaluate: list head).
			list := list tail].
	result := receiver perform: selector withArguments: arguments asArray.
	^result! !

!LispInterpreter methodsFor: 'fsubr functions' stamp: ''!
fsubrSetq: arguList 
	| list a1 a2 |
	list := arguList.
	a2 := LispNil null.
	[list isKindOf: LispCons]
		whileTrue: 
			[a1 := list head.
			(a1 isKindOf: Symbol)
				ifFalse: [^self fatal: 'unexpected variable ' , (self printString: a1) , ' for setq'].
			list := list tail.
			a2 := self evaluate: list head.
			self
				putprop: a1
				key: #apval
				value: a2.
			list := list tail].
	^a2! !

!LispInterpreter methodsFor: 'fsubr functions' stamp: ''!
fsubrSub: arguList 
	| v a |
	v := LispNil null.
	arguList
		do: 
			[:each | 
			a := self evaluate: each.
			(a isKindOf: Number)
				ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a) , ' for -'].
			v = LispNil null
				ifTrue: [v := a]
				ifFalse: [v := v - a]].
	^v! !

!LispInterpreter methodsFor: 'fsubr functions' stamp: ''!
fsubrWhile: arguList 
	| predicate do executions result |
	predicate := arguList head.
	do := arguList tail head.
	executions := arguList tail tail.
	do = #do ifFalse: [^self fatal: 'unexpected format for while'].
	result := LispNil null.
	[(self evaluate: predicate)
		= LispNil null]
		whileFalse: [executions do: [:each | result := self evaluate: each]].
	^result! !

!LispInterpreter methodsFor: 'interface opening' stamp: 'nishis 5/3/98 01:55'!
fullOpen
	"LispInterpreter fullOpen."

	| topView questionView transcriptView btns |
	topView := StandardSystemView new
			model: self;
			backgroundColor: Color white;
			label: 'Lisp';
			minimumSize: 250 @ 300.
	questionView := PluggableStringHolderView
			on: self
			aspect: #contents
			change: #contents:
			menu: #menu.
	questionView
			window: (0 @ 0 extent: 250 @ 100);
			borderWidth: 2;
			backgroundColor: Color lightYellow.
	transcriptView := PluggableTextView new
			model: self textCollector;
			window: (0 @ 0 extent: 250 @ 200);
			backgroundColor: Color white;
			borderWidthLeft: 2 right: 2 top: 0 bottom: 2. 

	btns := self buttonsView: questionView controller.
	topView addSubView: btns.
	topView addSubView: questionView below: btns.
	topView addSubView: transcriptView below: questionView.
	^ topView controller open
! !

!LispInterpreter methodsFor: 'property access' stamp: ''!
getprop: identifier key: key 
	| value |
	lispTable intern: identifier.
	value := lispTable getprop: identifier key: key.
	value isNil ifTrue: [^LispNil null].
	^value! !

!LispInterpreter methodsFor: 'initialize-release' stamp: 'nishis 4/25/98 04:08'!
initialize
	super initialize.
	lispTable := LispTable new.
	bindStack := OrderedCollection new.
	failBlock := [:errorMessage | self error: 'error' with: errorMessage].
	self initializeSubrFunctions.
	self initializeFsubrFunctions.
	self initializeExprFunctions.
	self initializeFexprFunctions.
	textCollector := TranscriptStream new! !

!LispInterpreter methodsFor: 'initialize-release' stamp: ''!
initializeExprFunctions
	self evaluateTopLevel: (LispParser parse: '

	% Expr Functions
	(progn

		% ++
		(defun ++ lambda (x)
			(+ x 1))

		% --
		(defun -- lambda (x)
			(- x 1))

		% assoc
		(defun assoc lambda (x a)
			(cond
				((null a) nil)
				((equal x (car (car a))) (car a))
				(t (assoc x (cdr a)))))

		% copy
		(defun copy lambda (x)
			(cond
				((null x) nil)
				(t (cons (car x) (copy (cdr x))))))

		% mapc
		(defun mapc lambda (f x)
			(cond
				((null x) nil)
				(t (progn
					(eval (cons f (cons `(car x) nil)))
					(mapc f (cdr x))))))

		% mapcar
		(defun mapcar lambda (f x)
			(cond
				((null x) nil)
				(t (cons
					(eval (cons f (cons `(car x) nil)))
					(mapcar f (cdr x))))))
			
		) % end

	')! !

!LispInterpreter methodsFor: 'initialize-release' stamp: ''!
initializeFexprFunctions
	self evaluateTopLevel: (LispParser parse: '

	% Expr Functions
	(progn

		% and
		(defun and nlambda (x)
			(do	(list)
				(setq list x)
				(while
					(if (null list) then nil else (eval (car list)))
					do
					(setq list (cdr list)))
				(if (null list) then t else nil)))

		% list
		(defun list nlambda (x)
			(mapcar `eval x))

		% or
		(defun or nlambda (x)
			(do	(list)
				(setq list x)
				(while
					(if (null list) then nil else (not (eval (car list))))
					do
					(setq list (cdr list)))
				(if (null list) then nil else t)))

		) % end

	')! !

!LispInterpreter methodsFor: 'initialize-release' stamp: ''!
initializeFsubrFunctions
	self setFsubrFunc: #(#* #fsubrMlt:).
	self setFsubrFunc: #(#+ #fsubrAdd:).
	self setFsubrFunc: #(#- #fsubrSub:).
	self setFsubrFunc: #(#/ #fsubrDiv:).
	self setFsubrFunc: #(#// #fsubrIdiv:).
	self setFsubrFunc: #(#cond #fsubrCond:).
	self setFsubrFunc: #(#defun #fsubrDefun:).
	self setFsubrFunc: #(#do #fsubrDo:).
	self setFsubrFunc: #(#if #fsubrIf:).
	self setFsubrFunc: #(#progn #fsubrProgn:).
	self setFsubrFunc: #(#quote #fsubrQuote:).
	self setFsubrFunc: #(#read #fsubrRead:).
	self setFsubrFunc: #(#repeat #fsubrRepeat:).
	self setFsubrFunc: #(#send #fsubrSend:).
	self setFsubrFunc: #(#setq #fsubrSetq:).
	self setFsubrFunc: #(#while #fsubrWhile:).
	self setFsubrFunc: #(#\\ #fsubrMod:).! !

!LispInterpreter methodsFor: 'initialize-release' stamp: ''!
initializeSubrFunctions
	self setSubrFunc: #(#< #subrLt: 2).
	self setSubrFunc: #(#<= #subrLe: 2).
	self setSubrFunc: #(#= #subrEqual: 2).
	self setSubrFunc: #(#== #subrEq: 2).
	self setSubrFunc: #(#> #subrGt: 2).
	self setSubrFunc: #(#>= #subrGe: 2).
	self setSubrFunc: #(#append #subrAppend: 2).
	self setSubrFunc: #(#atom #subrAtom: 1).
	self setSubrFunc: #(#car #subrCar: 1).
	self setSubrFunc: #(#cdr #subrCdr: 1).
	self setSubrFunc: #(#cons #subrCons: 2).
	self setSubrFunc: #(#consp #subrConsp: 1).
	self setSubrFunc: #(#dtpr #subrConsp: 1).
	self setSubrFunc: #(#doublep #subrDoublep: 1).
	self setSubrFunc: #(#eq #subrEq: 2).
	self setSubrFunc: #(#equal #subrEqual: 2).
	self setSubrFunc: #(#eval #subrEval: 1).
	self setSubrFunc: #(#exprs #subrExprs 0).
	self setSubrFunc: #(#fexprs #subrFexprs 0).
	self setSubrFunc: #(#floatp #subrFloatp: 1).
	self setSubrFunc: #(#fsubrs #subrFsubrs 0).
	self setSubrFunc: #(#gc #subrGc 0).
	self setSubrFunc: #(#gensym #subrGensym 0).
	self setSubrFunc: #(#getprop #subrGetprop: 2).
	self setSubrFunc: #(#integerp #subrIntegerp: 1).
	self setSubrFunc: #(#last #subrLast: 1).
	self setSubrFunc: #(#length #subrLength: 1).
	self setSubrFunc: #(#listp #subrListp: 1).
	self setSubrFunc: #(#member #subrMember: 2).
	self setSubrFunc: #(#memq #subrMemq: 2).
	self setSubrFunc: #(#nconc #subrNconc: 2).
	self setSubrFunc: #(#neq #subrNeq: 2).
	self setSubrFunc: #(#nequal #subrNequal: 2).
	self setSubrFunc: #(#not #subrNull: 1).
	self setSubrFunc: #(#nth #subrNth: 2).
	self setSubrFunc: #(#null #subrNull: 1).
	self setSubrFunc: #(#numberp #subrNumberp: 1).
	self setSubrFunc: #(#oblist #subrOblist 0).
	self setSubrFunc: #(#pp #subrPp: 1).
	self setSubrFunc: #(#princ #subrPrinc: 1).
	self setSubrFunc: #(#print #subrPrint: 1).
	self setSubrFunc: #(#putprop #subrPutprop: 3).
	self setSubrFunc: #(#remprop #subrRemprop: 2).
	self setSubrFunc: #(#reverse #subrReverse: 1).
	self setSubrFunc: #(#rplaca #subrRplaca: 2).
	self setSubrFunc: #(#rplacd #subrRplacd: 2).
	self setSubrFunc: #(#stringp #subrStringp: 1).
	self setSubrFunc: #(#subrs #subrSubrs 0).
	self setSubrFunc: #(#symbolp #subrSymbolp: 1).
	self setSubrFunc: #(#terpri #subrTerpri 0).
	self setSubrFunc: #(#~= #subrNequal: 2).
	self setSubrFunc: #(#~~ #subrNeq: 2)! !

!LispInterpreter methodsFor: 'menu messages' stamp: 'nishis 5/3/98 10:47'!
inspectIt: aText from: aController 
	| result |
	self acceptWith: aText.
	"see BrowserView browseFullForClass: PluggableStringHolderController method: #sendMenuMessageToModel:."
	result := self doIt: aText from: aController.
	result inspect.
	^result! !

!LispInterpreter methodsFor: 'menu messages' stamp: ''!
investigate
	self inspect! !

!LispInterpreter methodsFor: 'evaluating' stamp: ''!
listEvaluate: sExpression 
	| funcName arguList funcBody |
	sExpression null ifTrue: [^sExpression].
	funcName := sExpression head.
	arguList := sExpression tail.
	(funcName isKindOf: LispCons)
		ifTrue: 
			[funcBody := funcName.
			funcBody head = #lambda
				ifTrue: 
					[funcBody := LispCons head: #lambda tail: funcBody.
					^self exprEval: funcBody arguList: arguList].
			funcBody head = #nlambda
				ifTrue: 
					[funcBody := LispCons head: #nlambda tail: funcBody.
					^self fexprEval: funcBody arguList: arguList].
			^self fatal: 'unexpected function ' , (self printString: funcBody)].
	(funcName isKindOf: Symbol)
		ifFalse: [^self fatal: 'null function ' , (self printString: funcName)].
	funcBody := self getprop: funcName key: #fexpr.
	funcBody = LispNil null ifFalse: [^self fexprEval: funcBody arguList: arguList].
	funcBody := self getprop: funcName key: #expr.
	funcBody = LispNil null ifFalse: [^self exprEval: funcBody arguList: arguList].
	funcBody := self getprop: funcName key: #fsubr.
	funcBody = LispNil null ifFalse: [^self fsubrEval: funcBody arguList: arguList].
	funcBody := self getprop: funcName key: #subr.
	funcBody = LispNil null ifFalse: [^self subrEval: funcBody arguList: arguList].
	^self fatal: 'undefined function ' , (self printString: funcName)! !

!LispInterpreter methodsFor: 'pluggable messages' stamp: 'nishis 5/3/98 11:10'!
menu
	| collection |
	selectionMenu isNil
		ifTrue:
			[collection := OrderedCollection new.
			collection add: 'find...' -> #find.
			collection add: 'find again' -> #findAgain.
			collection add: 'set search string' -> #setSearchString.
			collection add: nil.
			collection add: 'do again' -> #again.
			collection add: 'undo' -> #undo.
			collection add: nil.
			collection add: 'copy' -> #copySelection.
			collection add: 'cut' -> #cut.
			collection add: 'paste' -> #paste.
			collection add: nil.
			collection add: 'do it' -> #doIt:from:.
			"collection add: 'print it' -> #printIt."
			collection add: 'inspect it' -> #inspectIt:from:.
			collection add: nil.
			collection add: 'accept' -> #accept.
			collection add: 'cancel' -> #cancel.
			collection add: nil.
			collection add: '--- lisp ---' -> #toBeIgnored.	
			collection add: 'gc' -> #subrGc.	
			collection add: 'investigate' -> #investigate.	
			selectionMenu := self makeSelectionMenu: collection].
	^ selectionMenu! !

!LispInterpreter methodsFor: 'interface opening' stamp: 'nishis 5/3/98 01:55'!
open
	"LispInterpreter open."

	| topView questionView transcriptView |
	topView := StandardSystemView new
			model: self;
			backgroundColor: Color white;
			label: 'Lisp';
			minimumSize: 250 @ 300.
	questionView := PluggableStringHolderView
			on: self
			aspect: #contents
			change: #contents:
			menu: #menu.
	questionView
			window: (0 @ 0 extent: 250 @ 100);
			borderWidth: 2;
			backgroundColor: Color lightYellow.
	transcriptView := PluggableTextView new
			model: self textCollector;
			window: (0 @ 0 extent: 250 @ 200);
			backgroundColor: Color white;
			borderWidthLeft: 2 right: 2 top: 0 bottom: 2. 

	topView addSubView: questionView.
	topView addSubView: transcriptView below: questionView.
	^ topView controller open
! !

!LispInterpreter methodsFor: 'printing' stamp: ''!
printString: anObject 
	^self class printString: anObject! !

!LispInterpreter methodsFor: 'property access' stamp: ''!
putprop: identifier key: key value: value 
	lispTable intern: identifier.
	^lispTable
		putprop: identifier
		key: key
		value: value! !

!LispInterpreter methodsFor: 'property access' stamp: ''!
remprop: identifier key: key 
	| value |
	lispTable intern: identifier.
	value := lispTable remprop: identifier key: key.
	value isNil ifTrue: [^LispNil null].
	^value! !

!LispInterpreter methodsFor: 'private' stamp: ''!
setFsubrFunc: bodyArray 
	self
		putprop: (bodyArray at: 1) asSymbol
		key: #fsubr
		value: (LispCons list: bodyArray)! !

!LispInterpreter methodsFor: 'private' stamp: ''!
setSubrFunc: bodyArray 
	self
		putprop: (bodyArray at: 1) asSymbol
		key: #subr
		value: (LispCons list: bodyArray)! !

!LispInterpreter methodsFor: 'private' stamp: ''!
skipSeparators: stream 
	| char |
	[stream atEnd not
		and: 
			[char := stream peek.
			(LispScannerTable scannerTable at: char asInteger)
				= #xDelimiter]]
		whileTrue: [stream next]! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrAppend: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: LispCons)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for append'].
	(a2 isKindOf: LispCons)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for append'].
	^a1 append: a2! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrAtom: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	(a1 isKindOf: LispCons) ifFalse: [^#t].
	^LispNil null! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrCar: arguArray 
	| list |
	list := arguArray at: 1.
	(list isKindOf: LispCons)
		ifTrue: [^list head].
	(list isKindOf: LispNil)
		ifTrue: [^LispNil null].
	^self fatal: 'unexpected argument ' , (self printString: list) , ' for car'! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrCdr: arguArray 
	| list |
	list := arguArray at: 1.
	(list isKindOf: LispCons)
		ifTrue: [^list tail].
	(list isKindOf: LispNil)
		ifTrue: [^LispNil null].
	^self fatal: 'unexpected argument ' , (self printString: list) , ' for cdr'! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrCons: arguArray 
	^LispCons head: (arguArray at: 1)
		tail: (arguArray at: 2)! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrConsp: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	(a1 isKindOf: LispCons)
		ifTrue: [^#t].
	^LispNil null! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrDoublep: arguArray 
	| a1 |
	^self subrFloatp: arguArray
	"a1 := arguArray at: 1.
	(a1 isKindOf: Double)
		ifTrue: [^#t].
	^LispNil null"! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrEq: arguArray 
	| bool |
	(arguArray at: 1)
		== (arguArray at: 2)
		ifTrue: [bool := #t]
		ifFalse: [bool := LispNil null].
	^bool! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrEqual: arguArray 
	| bool |
	(arguArray at: 1)
		= (arguArray at: 2)
		ifTrue: [bool := #t]
		ifFalse: [bool := LispNil null].
	^bool! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrEval: arguArray 
	^self evaluate: (arguArray at: 1)! !

!LispInterpreter methodsFor: 'func eval' stamp: ''!
subrEval: funcBody arguList: arguList 
	| funcName messageSelector arguCount arguBuffer list |
	funcName := funcBody head.
	messageSelector := funcBody tail head.
	arguCount := funcBody tail tail head.
	arguBuffer := OrderedCollection new.
	list := arguList.
	arguCount
		timesRepeat: 
			[list null ifTrue: [^self fatal: 'too few arguments for ' , (self printString: funcName)].
			arguBuffer add: (self evaluate: list head).
			list := list tail].
	list null not ifTrue: [^self fatal: 'too mary arguments for ' , (self printString: funcName)].
	arguBuffer isEmpty
		ifTrue: [^self perform: messageSelector]
		ifFalse: [^self perform: messageSelector with: arguBuffer asArray]! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrExprs
	| list |
	list := LispNil null.
	self subrOblist reverse do: [:id | (self getprop: id key: #expr)
			= LispNil null ifFalse: [list := LispCons head: id tail: list]].
	^list! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrFexprs
	| list |
	list := LispNil null.
	self subrOblist reverse do: [:id | (self getprop: id key: #fexpr)
			= LispNil null ifFalse: [list := LispCons head: id tail: list]].
	^list! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrFloatp: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	(a1 isKindOf: Float)
		ifTrue: [^#t].
	^LispNil null! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrFsubrs
	| list |
	list := LispNil null.
	self subrOblist reverse do: [:id | (self getprop: id key: #fsubr)
			= LispNil null ifFalse: [list := LispCons head: id tail: list]].
	^list! !

!LispInterpreter methodsFor: 'subr functions' stamp: 'nishis 4/20/98 23:40'!
subrGc
	"ObjectMemory globalCompactingGC."
	Transcript cr; show: 'garbage collecting: ', (Smalltalk garbageCollect) asString, ' bytes are available.'.
	^#t! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrGe: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: LispList)
		ifTrue: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for >='].
	(a2 isKindOf: LispList)
		ifTrue: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for >='].
	a1 >= a2 ifTrue: [^#t].
	^LispNil null! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrGensym
	| clock |
	(Delay forMilliseconds: 1) wait.
	clock := Time millisecondClockValue.
	^('id' , clock printString) asSymbol! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrGetprop: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: Symbol)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for getprop'].
	(a2 isKindOf: Symbol)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for getprop'].
	^self getprop: a1 key: a2! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrGt: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: LispList)
		ifTrue: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for >'].
	(a2 isKindOf: LispList)
		ifTrue: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for >'].
	a1 > a2 ifTrue: [^#t].
	^LispNil null! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrIntegerp: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	(a1 isKindOf: Integer)
		ifTrue: [^#t].
	^LispNil null! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrLast: arguArray 
	| list |
	list := arguArray at: 1.
	(list isKindOf: LispCons)
		ifTrue: [^list last].
	^self fatal: 'unexpected argument ' , (self printString: list) , ' for last'! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrLe: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: LispList)
		ifTrue: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for <='].
	(a2 isKindOf: LispList)
		ifTrue: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for <='].
	a1 <= a2 ifTrue: [^#t].
	^LispNil null! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrLength: arguArray 
	| list |
	list := arguArray at: 1.
	(list isKindOf: LispCons)
		ifTrue: [^list length].
	^self fatal: 'unexpected argument ' , (self printString: list) , ' for length'! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrListp: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	(a1 isKindOf: LispList)
		ifTrue: [^#t].
	^LispNil null! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrLt: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: LispList)
		ifTrue: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for <'].
	(a2 isKindOf: LispList)
		ifTrue: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for <'].
	a1 < a2 ifTrue: [^#t].
	^LispNil null! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrMember: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a2 isKindOf: LispCons)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for member'].
	^a2 member: a1! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrMemq: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a2 isKindOf: LispCons)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for memq'].
	^a2 memq: a1! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrNconc: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: LispCons)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for nconc'].
	a1 nconc: a2.
	^a1! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrNeq: arguArray 
	| bool |
	((arguArray at: 1)
		== (arguArray at: 2)) not
		ifTrue: [bool := #t]
		ifFalse: [bool := LispNil null].
	^bool! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrNequal: arguArray 
	| bool |
	((arguArray at: 1)
		= (arguArray at: 2)) not
		ifTrue: [bool := #t]
		ifFalse: [bool := LispNil null].
	^bool! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrNth: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: Number)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for nth'].
	(a2 isKindOf: LispCons)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for nth'].
	^a2 nth: a1! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrNull: arguArray 
	((arguArray at: 1)
		isMemberOf: LispNil)
		ifTrue: [^#t].
	^LispNil null! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrNumberp: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	(a1 isKindOf: Number)
		ifTrue: [^#t].
	^LispNil null! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrOblist
	| list |
	list := LispNil null.
	lispTable identifiers reverseDo: [:each | list := LispCons head: each tail: list].
	^list! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrPp: arguArray 
	| a pretty |
	a := arguArray at: 1.
	pretty := a ppString.
	textCollector show: pretty.
	^a! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrPrinc: arguArray 
	| a |
	a := arguArray at: 1.
	(a isKindOf: String)
		ifTrue: [textCollector show: a]
		ifFalse: [textCollector show: (self printString: a)].
	^a! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrPrint: arguArray 
	| a |
	a := self subrPrinc: arguArray.
	textCollector cr.
	^a! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrPutprop: arguArray 
	| a1 a2 a3 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	a3 := arguArray at: 3.
	(a1 isKindOf: Symbol)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for putprop'].
	(a2 isKindOf: Symbol)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for putprop'].
	^self
		putprop: a1
		key: a2
		value: a3! !

!LispInterpreter methodsFor: 'subr functions' stamp: 'nishis 4/21/98 01:22'!
subrRead
	| answer theClipboardText |
	theClipboardText := Smalltalk clipboardText.
	Smalltalk clipboardText: String new.
	answer := FillInTheBlank request: '          Please type S expression.          ' initialAnswer: ''.
	Smalltalk clipboardText: theClipboardText.
	(answer isNil or: [answer isEmpty])
		ifTrue: 
			[^LispNil null].
	^LispParser parse: (ReadStream on: answer)! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrRemprop: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: Symbol)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for remprop'].
	(a2 isKindOf: Symbol)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for remprop'].
	^self remprop: a1 key: a2! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrReverse: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	(a1 isKindOf: LispCons)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a1) , ' for reverse'].
	^a1 reverse! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrRplaca: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: LispCons)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for rplaca'].
	a1 rplaca: a2.
	^a1! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrRplacd: arguArray 
	| a1 a2 |
	a1 := arguArray at: 1.
	a2 := arguArray at: 2.
	(a1 isKindOf: LispCons)
		ifFalse: [^self fatal: 'unexpected argument ' , (self printString: a2) , ' for rplacd'].
	a1 rplacd: a2.
	^a1! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrStringp: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	((a1 isKindOf: String)
		and: [(a1 isKindOf: Symbol) not])
		ifTrue: [^#t].
	^LispNil null! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrSubrs
	| list |
	list := LispNil null.
	self subrOblist reverse do: [:id | (self getprop: id key: #subr)
			= LispNil null ifFalse: [list := LispCons head: id tail: list]].
	^list! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrSymbolp: arguArray 
	| a1 |
	a1 := arguArray at: 1.
	(a1 isKindOf: Symbol) ifTrue: [^#t].
	^LispNil null! !

!LispInterpreter methodsFor: 'subr functions' stamp: ''!
subrTerpri
	textCollector cr.
	^#t! !

!LispInterpreter methodsFor: 'accessing' stamp: ''!
textCollector
	^textCollector! !

!LispInterpreter methodsFor: 'adaptor' stamp: 'nishis 5/3/98 11:05'!
textValue
	textValue isNil ifTrue: [textValue := String new].
	^textValue! !

!LispInterpreter methodsFor: 'menu messages' stamp: 'nishis 4/25/98 03:41'!
toBeIgnored! !

!LispInterpreter methodsFor: 'shallow binding' stamp: ''!
unbind
	| assoc |
	
	[assoc := bindStack removeLast.
	assoc key notNil]
		whileTrue: [assoc value notNil
				ifTrue: [self
						putprop: assoc key
						key: #apval
						value: assoc value]
				ifFalse: [self remprop: assoc key key: #apval]]! !

!PluggableStringHolderModel class methodsFor: 'examples' stamp: ''!
example1
	"PluggableStringHolderModel example1."

	| aPluggableStringHolderModel aPluggableStringHolderView aStandardSystemView |
	aPluggableStringHolderModel := PluggableStringHolderModel new.
	aPluggableStringHolderView := PluggableStringHolderView
			on: aPluggableStringHolderModel
			aspect: #contents
			change: #contents:
			menu: #menu.
	aPluggableStringHolderView borderWidth: 1.
	aStandardSystemView := StandardSystemView new.
	aStandardSystemView borderWidth: 1.
	aStandardSystemView model: aPluggableStringHolderModel.
	aStandardSystemView addSubView: aPluggableStringHolderView.
	aStandardSystemView label: 'Pluggable String Holder'.
	aStandardSystemView minimumSize: 100 @ 50.
	aStandardSystemView controller open! !

!PluggableStringHolderModel methodsFor: 'initialize-release' stamp: ''!
defaultBackgroundColor
	^#white! !

!PluggableStringHolderModel methodsFor: 'menu messages' stamp: ''!
doSomething
	self textCollector cr; show: thisContext printString! !

!PluggableStringHolderModel methodsFor: 'menu messages' stamp: ''!
doSomething: selectedString
	self textCollector cr; show: thisContext printString.
	self textCollector cr; tab; show: '1st argument is ' , selectedString printString! !

!PluggableStringHolderModel methodsFor: 'menu messages' stamp: ''!
doSomething: selectedString with: wholeString
	self textCollector cr; show: thisContext printString.
	self textCollector cr; tab; show: '1st argument is ' , selectedString printString.
	self textCollector cr; tab; show: '2nd argument is ' , wholeString printString! !

!PluggableStringHolderModel methodsFor: 'menu messages' stamp: ''!
doSomething: selectedString with: wholeString with: aController
	self textCollector cr; show: thisContext printString.
	self textCollector cr; tab; show: '1st argument is ' , selectedString printString.
	self textCollector cr; tab; show: '2nd argument is ' , wholeString printString.
	self textCollector cr; tab; show: '3rd argument is ' , aController printString! !

!PluggableStringHolderModel methodsFor: 'private' stamp: ''!
makeSelectionMenu: aCollection
	| labels lines selectors |
	aCollection isEmpty ifTrue: [^nil].
	labels := OrderedCollection new.
	lines := OrderedCollection new.
	selectors := OrderedCollection new.
	aCollection do: [:association |
		association isNil
			ifTrue: [lines add: labels size]
			ifFalse:
				[labels add: association key.
				selectors add: association value]].
	^SelectionMenu 
		labelList: labels asArray
		lines: lines asArray
		selections: selectors asArray! !

!PluggableStringHolderModel methodsFor: 'pluggable messages' stamp: ''!
menu
	| collection |
	selectionMenu isNil
		ifTrue:
			[collection := OrderedCollection new.
			collection add: 'find...' -> #find.
			collection add: 'find again' -> #findAgain.
			collection add: 'set search string' -> #setSearchString.
			collection add: nil.
			collection add: 'do again' -> #again.
			collection add: 'undo' -> #undo.
			collection add: nil.
			collection add: 'copy' -> #copySelection.
			collection add: 'cut' -> #cut.
			collection add: 'paste' -> #paste.
			collection add: nil.
			collection add: 'do it' -> #doIt.
			collection add: 'print it' -> #printIt.
			collection add: 'inspect it' -> #inspectIt.
			collection add: nil.
			collection add: 'accept' -> #accept.
			collection add: 'cancel' -> #cancel.
			collection add: nil.
			collection add: 'doSomething' -> #doSomething.
			collection add: 'doSomething:' -> #doSomething:.
			collection add: 'doSomething:with:' -> #doSomething:with:.
			collection add: 'doSomething:with:with:' -> #doSomething:with:with:.
			selectionMenu := self makeSelectionMenu: collection].
	^selectionMenu! !

!PluggableStringHolderModel methodsFor: 'accessing' stamp: 'nishis 4/26/98 21:06'!
textCollector
	^Transcript! !

Model subclass: #PluggableYAListModel
	instanceVariableNames: 'selectionMenu list selection listIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lisp-Pluggable-Support'!

!PluggableYAListModel commentStamp: 'nishis 6/28/1998 09:39' prior: 0!
#
# This file is encoded in (so called) SHIFT-JIS,
# line delimitor is CR.
#
PluggableYAListModel (Pluggable-StringHolderMVC)
Goodies for Squeak 2.0
!

!PluggableYAListModel class methodsFor: 'example' stamp: 'nishis 5/3/98 09:00'!
example
	"PluggableYAListModel example."

	| model listView topView |
	model := self new initialize.
	model list: model musicBands.
	listView :=
		PluggableYAListView on: model
			list: #musicBands
			selected: #selection
			changeSelected: #selection:
			menu: #menu.
	topView := StandardSystemView new
		label: 'PluggableYAListModel Test';
		minimumSize: 200@200;
		borderWidth: 1;
		addSubView: listView.
	topView controller open.! !

!PluggableYAListModel class methodsFor: 'instance creation' stamp: 'nishis 4/30/98 23:26'!
new
	^super new initialize! !

!PluggableYAListModel class methodsFor: 'saving' stamp: 'nishis 6/11/1998 22:42'!
save
	"PluggableYAListModel save."

	| fileName classCollection aStream timeStamp |
	fileName := self saveFilename.
	classCollection := self saveClasses.
	aStream := FileStream newFileNamed: fileName asFileName.
	aStream reset.
	Cursor write
		showWhile: 
			[timeStamp := Date today mmddyyyy, ' ' , Time now asString.
			aStream cr.
			aStream nextChunkPut: timeStamp printString.
			aStream cr; cr.
			(self comment isNil or: [self comment isEmpty])
				ifFalse: 
					[aStream nextChunkPut: (self comment asString) printString.
					aStream cr; cr].
			classCollection
				do: 
					[:aClass | 
					aStream nextChunkPut: aClass definition.
					aStream cr; cr].
			classCollection
				do: 
					[:aClass | 
					aStream nextPut: Character newPage.
					aStream cr.
					aClass fileOutOn: aStream.
					aStream cr]].
	aStream close.
	^classCollection! !

!PluggableYAListModel class methodsFor: 'saving' stamp: 'nishis 5/3/98 11:29'!
saveAllClasses
	"PluggableYAListModel saveAllClasses."

	| patternCollection classCollection string something |
	patternCollection := #('*').
	classCollection := SystemOrganization superclassOrder: self category.
	classCollection := classCollection
				select: 
					[:aClass | 
					string := aClass name asString.
					something := patternCollection detect: [:it | it match: string]
								ifNone: [nil].
					something notNil].
	^classCollection! !

!PluggableYAListModel class methodsFor: 'saving' stamp: 'nishis 5/3/98 11:26'!
saveClasses
	"PluggableYAListModel saveClasses."

	| patternCollection classCollection string something |
	patternCollection := #('*Pluggable*').
	classCollection := SystemOrganization superclassOrder: self category.
	classCollection := classCollection
				select: 
					[:aClass | 
					string := aClass name asString.
					something := patternCollection detect: [:it | it match: string]
								ifNone: [nil].
					something notNil].
	^classCollection! !

!PluggableYAListModel class methodsFor: 'saving' stamp: 'nishis 5/3/98 12:27'!
saveFilename
	^'Plugable.st'! !

!PluggableYAListModel methodsFor: 'list' stamp: 'nishis 5/3/98 08:25'!
changed: what
	super changed: what! !

!PluggableYAListModel methodsFor: 'example' stamp: 'nishis 5/3/98 08:19'!
example
	"PluggableYAListModel example"

	| model listView1 topView listView2 |
	model _ self new initialize.
	listView1 _
		PluggableYAListView on: model
			list: #musicTypes
			selected: #musicType
			changeSelected: #musicType:
			menu: #musicTypeMenu
			keystroke: #musicTypeKeystroke:.

	listView2 _
		PluggableYAListView on: model
			list: #artists
			selected: #artist
			changeSelected: #artist:
			menu: nil
			keystroke: #artistKeystroke:.

	topView _ StandardSystemView new
		label: 'Pluggable Test';
		minimumSize: 300@200;
		borderWidth: 1;
		addSubView: listView1;
		addSubView: listView2 toRightOf: listView1.
	topView controller open.! !

!PluggableYAListModel methodsFor: 'initialize-release' stamp: 'nishis 5/3/98 08:26'!
initialize
	list := nil.
	selection := nil.
	listIndex := 0! !

!PluggableYAListModel methodsFor: 'accessing' stamp: 'nishis 4/30/98 22:46'!
list
	^list! !

!PluggableYAListModel methodsFor: 'accessing' stamp: 'nishis 4/30/98 22:48'!
list: aColection
	list := aColection.
	listIndex := 0.
	self changed: #list! !

!PluggableYAListModel methodsFor: 'accessing' stamp: 'nishis 4/30/98 22:48'!
listIndex
	^listIndex! !

!PluggableYAListModel methodsFor: 'private' stamp: 'nishis 5/3/98 08:23'!
makeSelectionMenu: aCollection
	| labels lines selectors |
	aCollection isEmpty ifTrue: [^nil].
	labels := OrderedCollection new.
	lines := OrderedCollection new.
	selectors := OrderedCollection new.
	aCollection do: [:association |
		association isNil
			ifTrue: [lines add: labels size]
			ifFalse:
				[labels add: association key.
				selectors add: association value]].
	^SelectionMenu 
		labelList: labels asArray
		lines: lines asArray
		selections: selectors asArray! !

!PluggableYAListModel methodsFor: 'pluggable messages' stamp: 'nishis 5/3/98 08:57'!
menu
	| collection |
	selectionMenu isNil
		ifTrue:
			[collection := OrderedCollection new.
			collection add: 'update' -> #update.
			collection add: 'update:' -> #update:.
			collection add: 'update:with:' -> #update:with:.
			collection add: 'update:with:with:' -> #update:with:with:.
			selectionMenu := self makeSelectionMenu: collection].
	^selectionMenu! !

!PluggableYAListModel methodsFor: 'private' stamp: 'nishis 5/3/98 08:29'!
musicBands
	^#('Hatfield and the North' 'National Health' 'Soft Machine' 'Gong').
! !

!PluggableYAListModel methodsFor: 'accessing' stamp: 'nishis 4/30/98 22:48'!
selection
	^ listIndex = 0 
		ifFalse: [list at: listIndex]
		ifTrue: [nil]! !

!PluggableYAListModel methodsFor: 'accessing' stamp: 'nishis 5/3/98 09:06'!
selection: aSelection
	| index |
	(index _ list indexOf: aSelection) = 0 
		ifFalse: [listIndex == index 
					ifFalse: [self toggleListIndex: index]
		ifTrue: [self changed: #listIndex]].! !

!PluggableYAListModel methodsFor: 'list' stamp: 'nishis 5/3/98 08:25'!
toggleListIndex: aNumber
	"What to do when the user chooses an item"
	listIndex == aNumber ifTrue: [listIndex _ 0]
		ifFalse: [listIndex _ aNumber].
	self changed: #listIndex! !

!PluggableYAListModel methodsFor: 'menu messages' stamp: 'nishis 5/3/98 09:11'!
update
	Transcript cr; show: thisContext printString.
	Transcript cr; tab; show: 'selection is ' , self selection printString! !

!PluggableYAListModel methodsFor: 'menu messages' stamp: 'nishis 5/3/98 09:11'!
update: selectedString
	Transcript cr; show: thisContext printString.
	Transcript cr; tab; show: 'selection is ' , self selection printString.
	Transcript cr; tab; show: '1st argument is ' , selectedString printString! !

!PluggableYAListModel methodsFor: 'menu messages' stamp: 'nishis 5/3/98 09:12'!
update: selectedString with: wholeString
	Transcript cr; show: thisContext printString.
	Transcript cr; tab; show: 'selection is ' , self selection printString.
	Transcript cr; tab; show: '1st argument is ' , selectedString printString.
	Transcript cr; tab; show: '2nd argument is ' , wholeString printString! !

!PluggableYAListModel methodsFor: 'menu messages' stamp: 'nishis 5/3/98 09:12'!
update: selectedString with: wholeString with: aController
	Transcript cr; show: thisContext printString.
	Transcript cr; tab; show: 'selection is ' , self selection printString.
	Transcript cr; tab; show: '1st argument is ' , selectedString printString.
	Transcript cr; tab; show: '2nd argument is ' , wholeString printString.
	Transcript cr; tab; show: '3rd argument is ' , aController printString! !

StringHolderView subclass: #PluggableStringHolderView
	instanceVariableNames: 'aspectSelector changeSelector menuSelector'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lisp-Pluggable-Support'!

!PluggableStringHolderView commentStamp: 'nishis 6/28/1998 09:39' prior: 0!
#
# This file is encoded in (so called) SHIFT-JIS,
# line delimitor is CR.
#
PluggableStringHolderView (Pluggable-StringHolderMVC)
Goodies for Squeak 1.31
This goodies for Squeak from Mr. AOKI
!

!PluggableStringHolderView class methodsFor: 'instance creation' stamp: 'nishis 5/2/98 01:46'!
new
	^super new initialize! !

!PluggableStringHolderView class methodsFor: 'instance creation' stamp: ''!
on: aModel aspect: aspectSymbol change: changeSymbol menu: menuSymbol
	| aView |
	aView := self new.
	aView
		on: aModel
		aspect: aspectSymbol
		change: changeSymbol
		menu: menuSymbol.
	^aView! !

!PluggableStringHolderView methodsFor: 'pluggable support' stamp: 'nishis 5/2/98 01:52'!
contents
	^self model perform: aspectSelector! !

!PluggableStringHolderView methodsFor: 'pluggable support' stamp: ''!
contents: aString
	^self model perform: changeSelector with: aString! !

!PluggableStringHolderView methodsFor: 'controller access' stamp: ''!
defaultControllerClass 
	^PluggableStringHolderController! !

!PluggableStringHolderView methodsFor: 'pluggable support' stamp: ''!
menu
	^self model perform: menuSelector! !

!PluggableStringHolderView methodsFor: 'model access' stamp: ''!
model: aModel 
	self model: aModel controller: self controller.
	self editString: self contents! !

!PluggableStringHolderView methodsFor: 'private' stamp: ''!
on: aModel aspect: aspectSymbol change: changeSymbol menu: menuSymbol
	aspectSelector := aspectSymbol.
	changeSelector := changeSymbol.
	menuSelector := menuSymbol.
	self model: aModel! !

!PluggableStringHolderView methodsFor: 'updating' stamp: ''!
updateDisplayContents
	self editString: self contents.
	self displayView! !

StringHolderController subclass: #PluggableStringHolderController
	instanceVariableNames: 'localMenuSelectors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lisp-Pluggable-Support'!

!PluggableStringHolderController commentStamp: 'nishis 6/28/1998 09:39' prior: 0!
#
# This file is encoded in (so called) SHIFT-JIS,
# line delimitor is CR.
#
PluggableStringHolderController (Pluggable-StringHolderMVC)
Goodies for Squeak 1.31
This goodies for Squeak from Mr. AOKI
!

!PluggableStringHolderController methodsFor: 'menu messages' stamp: 'nishis 6/12/1998 02:11'!
accept 
	super accept.
	initialText := paragraph text copy.
	"view ifNotNil: [view accepted]."
	self view contents: paragraph string.
	"self unlockModel"! !

!PluggableStringHolderController methodsFor: 'initialize-release' stamp: ''!
initialize
	super initialize.
	localMenuSelectors := #(find findAgain setSearchString again undo copySelection cut paste doIt printIt inspectIt accept cancel)! !

!PluggableStringHolderController methodsFor: 'accessing' stamp: ''!
localMenuSelectors
	^localMenuSelectors! !

!PluggableStringHolderController methodsFor: 'accessing' stamp: ''!
localMenuSelectors: selectorArray
	localMenuSelectors := selectorArray! !

!PluggableStringHolderController methodsFor: 'pluggable support' stamp: ''!
sendMenuMessageToModel: selector
	| kind arguments |
	self controlTerminate.
	kind := selector numArgs.
	arguments := nil.
	kind = 0 ifTrue: [arguments := Array new].
	kind = 1 ifTrue: [arguments := Array with: self selection asString].
	kind = 2 ifTrue: [arguments := Array with: self selection asString with: paragraph asString].
	kind = 3 ifTrue: [arguments := Array with: self selection asString with: paragraph asString with: self].
	arguments notNil ifTrue: [self model perform: selector withArguments: arguments].
	self controlInitialize! !

!PluggableStringHolderController methodsFor: 'menu messages' stamp: 'nishis 4/26/98 21:15'!
yellowButtonActivity
	| menu selector |
	menu := self view menu.
	menu isNil
		ifTrue: [super controlActivity]
		ifFalse: 
			[selector := menu startUp.
			selector isNil 
				ifFalse:
					[(self localMenuSelectors includes: selector)
						ifTrue: [self menuMessageReceiver performMenuMessage: selector]
						ifFalse: [self sendMenuMessageToModel: selector]]]! !

SequenceableCollection subclass: #LispList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lisp-Interpreter'!

LispList subclass: #LispCons
	instanceVariableNames: 'head tail'
	classVariableNames: 'HorizontalLevel VerticalLevel'
	poolDictionaries: ''
	category: 'Lisp-Interpreter'!

!LispCons class methodsFor: 'instance creation' stamp: ''!
cell
	^super new head: self null tail: self null! !

!LispCons class methodsFor: 'utilities' stamp: ''!
classHierarchy: aClass 
	"LispCons classHierarchy: Number."

	| theClass list |
	aClass isMeta
		ifTrue: [theClass := aClass soleInstance]
		ifFalse: [theClass := aClass].
	list := self subclassHierarchy: theClass.
	(theClass allSuperclasses select: [:each | each isMeta not])
		do: [:each | list := self head: each name tail: (self head: list tail: self null)].
	^list! !

!LispCons class methodsFor: 'copyright' stamp: ''!
copyright
	^'Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.'! !

!LispCons class methodsFor: 'examples' stamp: ''!
example1
	"LispCons example1."

	| list |
	list := LispCons list: #(1 2 3 4 5 6 7 8 9 10 ).
	Transcript cr; show: list printString.
	^list! !

!LispCons class methodsFor: 'examples' stamp: ''!
example2
	"LispCons example2."

	| null list |
	null := LispCons null.
	list := LispCons list: #(1 2 ).
	list := LispCons head: list tail: null.
	list := LispCons head: list tail: null.
	Transcript cr; show: list printString.
	^list! !

!LispCons class methodsFor: 'examples' stamp: ''!
example3
	"LispCons example3."

	| x y z |
	x := LispCons list: #(1 2 3 ).
	y := LispCons list: #(4 5 6 ).
	z := LispCons list: #(1 2 3 4 5 6 ).
	Transcript cr; show: '(setq x ''(1 2 3)) => ' , x printString.
	Transcript cr; show: '(setq y ''(4 5 6)) => ' , y printString.
	Transcript cr; show: '(setq z ''(1 2 3 4 5 6)) => ' , z printString.
	Transcript cr; show: '(append x y) => ' , (x append: y) printString.
	Transcript cr; show: '(length z) => ' , z length printString.
	Transcript cr; show: '(member 3 z) => ' , (z member: 3) printString.
	Transcript cr; show: '(nth 4 z) => ' , (z nth: 4) printString.
	^z! !

!LispCons class methodsFor: 'examples' stamp: ''!
example4
	"LispCons example4."

	| list |
	list := LispCons list: #(1 2 ).
	list := LispCons head: list tail: (LispCons list: #(3 4 )).
	list := LispCons head: list tail: (LispCons list: #(5 6 )).
	Transcript cr; show: list saveString.
	^list! !

!LispCons class methodsFor: 'examples' stamp: ''!
example5
	"LispCons example5."

	| list |
	list := LispCons loadFrom: '
			(PetriNet Aoki
				(Place p1 p2 p3 p4 p5)
				(Transition t1 t2 t3 t4 t5)
				(InputFunction
					(t1 p1 p2 p3 p4 p5)
					(t2 . p4)
					(t3 . p5))
				(OutputFunction
					(t1 p1 p2 p3 p4 p5)
					(t2 . p4)
					(t3 . p5))
				(Marking {#(1 2 3 4 5)})))'.
	Transcript cr; show: list saveString.
	^list! !

!LispCons class methodsFor: 'examples' stamp: ''!
example6
	"LispCons example6."

	| list |
	list := LispCons loadFrom: '(aaa bbb ccc)'.
	^list! !

!LispCons class methodsFor: 'examples' stamp: ''!
example7
	"LispCons example7."

	| list |
	list := LispCons loadFrom: ' `(`(1 2 `3) . `4 ) '.
	^list! !

!LispCons class methodsFor: 'examples' stamp: ''!
example8
	"LispCons example8."

	| list |
	list := LispCons loadFrom: ' ({100@100} {200@200} {Form fromUser}) '.
	^list! !

!LispCons class methodsFor: 'examples' stamp: ''!
example9
	"LispCons example9."

	LispCons null.
	LispCons cell.
	LispCons new: 5.
	LispCons head: 10 tail: (LispCons head: 20 tail: LispCons null).
	LispCons list: #(10 20 ).
	LispCons loadFrom: ' (10 {100@200} {Form fromUser}) '.
	(LispCons list: #(10 20 )) head.
	(LispCons list: #(10 20 )) tail.
	(LispCons list: #(10 20 ))
		append: (LispCons list: #(30 40 )).
	(LispCons list: #(10 20 )) last.
	(LispCons list: #(10 20 )) length.
	(LispCons list: #(10 20 ))
		nth: 2.
	(LispCons list: #(10 20 ))
		do: [:each | Transcript cr; show: each printString].
	(LispCons list: (0 to: 100)) printString.
	(LispCons list: (0 to: 100)) ppString.
	(LispCons list: (0 to: 100)) saveString! !

!LispCons class methodsFor: 'instance creation' stamp: ''!
head: headObject 
	^super new head: headObject tail: self null! !

!LispCons class methodsFor: 'instance creation' stamp: ''!
head: headObject tail: tailObject 
	^super new head: headObject tail: tailObject! !

!LispCons class methodsFor: 'level accessing' stamp: ''!
horizontalLevel
	HorizontalLevel isNil ifTrue: [HorizontalLevel := 50].
	^HorizontalLevel! !

!LispCons class methodsFor: 'level accessing' stamp: ''!
horizontalLevel: anInteger 
	HorizontalLevel := anInteger! !

!LispCons class methodsFor: 'class initialization' stamp: ''!
initialize
	"LispCons initialize."

	HorizontalLevel := VerticalLevel := nil! !

!LispCons class methodsFor: 'instance creation' stamp: ''!
list: anArray 
	| size list |
	size := anArray size.
	list := self null.
	size
		to: 1
		by: -1
		do: [:i | list := self head: (anArray at: i)
						tail: list].
	^list! !

!LispCons class methodsFor: 'instance creation' stamp: ''!
loadFrom: aStream 
	"by nishis, 1998/04/19 07:51"

	| list |
	list := LispParser parse: aStream.
	^list! !

!LispCons class methodsFor: 'instance creation' stamp: ''!
new
	^self cell! !

!LispCons class methodsFor: 'instance creation' stamp: ''!
null
	^LispNil null! !

!LispCons class methodsFor: 'utilities' stamp: ''!
subclassHierarchy: aClass 
	"LispCons subclassHierarchy: Number."

	| theClass list collection sub |
	aClass isMeta
		ifTrue: [theClass := aClass soleInstance]
		ifFalse: [theClass := aClass].
	list := self null.
	theClass subclasses isEmpty
		ifFalse: 
			[collection := SortedCollection sortBlock: [:x :y | x name > y name].
			collection addAll: (theClass subclasses select: [:each | each isMeta not]).
			collection
				do: 
					[:each | 
					sub := self subclassHierarchy: each.
					list := self head: sub tail: list]].
	^self head: theClass name tail: list! !

!LispCons class methodsFor: 'utilities' stamp: ''!
superclassHierarchy: aClass 
	"LispCons superclassHierarchy: Number."

	| theClass list |
	aClass isMeta
		ifTrue: [theClass := aClass soleInstance]
		ifFalse: [theClass := aClass].
	list := self head: theClass name tail: self null.
	(theClass allSuperclasses select: [:each | each isMeta not])
		do: [:each | list := self head: each name tail: (self head: list tail: self null)].
	^list! !

!LispCons class methodsFor: 'copyright' stamp: ''!
system
	^'Goodies'! !

!LispCons class methodsFor: 'copyright' stamp: ''!
version
	^'003'! !

!LispCons class methodsFor: 'level accessing' stamp: ''!
verticalLevel
	VerticalLevel isNil ifTrue: [VerticalLevel := 10].
	^VerticalLevel! !

!LispCons class methodsFor: 'level accessing' stamp: ''!
verticalLevel: anInteger 
	VerticalLevel := anInteger! !

!LispCons methodsFor: 'testing' stamp: ''!
= anObject 
	(anObject isKindOf: self class)
		ifFalse: [^false].
	self head = anObject head ifTrue: [^self tail = anObject tail].
	^false! !

!LispCons methodsFor: 'adding' stamp: ''!
add: newObject 
	^self nconc: (self class head: newObject tail: self class null)! !

!LispCons methodsFor: 'functions' stamp: ''!
append: list 
	(tail isKindOf: self class)
		ifFalse: [^self class head: head tail: list].
	^self class head: head tail: (tail append: list)! !

!LispCons methodsFor: 'accessing' stamp: ''!
at: indexInteger
	| count |
	count := 1.
	self
		mapcdr: 
			[:cdr | 
			indexInteger = count ifTrue: [^cdr head].
			count := count + 1].
	^self subscriptOutOfBoundsError: indexInteger! !

!LispCons methodsFor: 'accessing' stamp: ''!
at: indexInteger put: anObject 
	| count |
	count := 1.
	self
		mapcdr: 
			[:cdr | 
			indexInteger = count ifTrue: [^cdr head: anObject].
			count := count + 1].
	^self subscriptOutOfBoundsError: indexInteger! !

!LispCons methodsFor: 'enumerating' stamp: ''!
collect: aBlock 
	| list result |
	list := self.
	result := self class null.
	[list isKindOf: self class]
		whileTrue: 
			[result := self class head: (aBlock value: list head)
						tail: result.
			list := list tail].
	^result reverse! !

!LispCons methodsFor: 'enumerating' stamp: ''!
do: aBlock 
	| list |
	list := self.
	[list isKindOf: self class]
		whileTrue: 
			[aBlock value: list head.
			list := list tail]! !

!LispCons methodsFor: 'accessing' stamp: ''!
head
	^head! !

!LispCons methodsFor: 'accessing' stamp: ''!
head: anObject 
	^head := anObject! !

!LispCons methodsFor: 'accessing' stamp: ''!
head: headObject tail: tailObject 
	self head: headObject.
	self tail: tailObject! !

!LispCons methodsFor: 'functions' stamp: ''!
last
	| list |
	list := self class head: nil tail: self.
	self do: [:each | list := list tail].
	^list! !

!LispCons methodsFor: 'functions' stamp: ''!
length
	| count |
	count := 0.
	self do: [:each | count := count + 1].
	^count! !

!LispCons methodsFor: 'functions' stamp: ''!
mapcdr: aBlock 
	| list |
	list := self.
	[list isKindOf: self class]
		whileTrue: 
			[aBlock value: list.
			list := list tail]! !

!LispCons methodsFor: 'functions' stamp: ''!
member: anObject 
	| list |
	list := self.
	self do: 
		[:each | 
		each = anObject ifTrue: [^list].
		list := list tail].
	^self species null! !

!LispCons methodsFor: 'functions' stamp: ''!
memq: anObject 
	| list |
	list := self.
	self do: 
		[:each | 
		each == anObject ifTrue: [^list].
		list := list tail].
	^self species null! !

!LispCons methodsFor: 'functions' stamp: ''!
nconc: list 
	self last rplacd: list! !

!LispCons methodsFor: 'functions' stamp: ''!
nth: nth 
	| count list |
	nth <= 0 ifTrue: [^self species null].
	count := 1.
	list := self.
	list do: 
		[:each | 
		count >= nth ifTrue: [^each].
		count := count + 1].
	^self species null! !

!LispCons methodsFor: 'pretty printing' stamp: ''!
ppOn: aStream 
	self
		ppOn: aStream
		list: self
		position: 0.
	aStream cr! !

!LispCons methodsFor: 'pretty printing' stamp: ''!
ppOn: aStream list: list position: position 
	(list isKindOf: self class)
		ifFalse: [^self ppOn: aStream object: list].
	(list head isKindOf: self class) not
		ifTrue: 
			[aStream nextPutAll: '('.
			self ppOn: aStream object: list head.
			(list tail isKindOf: LispList)
				ifTrue: [self
						ppOn: aStream
						tail: list tail
						position: position + 1]
				ifFalse: 
					[aStream nextPutAll: ' . '.
					self ppOn: aStream object: list tail].
			aStream nextPutAll: ')']
		ifFalse: 
			[aStream nextPutAll: '('.
			self
				ppOn: aStream
				list: list head
				position: position + 1.
			(list tail isKindOf: self class)
				ifTrue: [(list tail head isKindOf: self class)
						ifTrue: 
							[aStream cr.
							self ppOn: aStream spaceAndTab: position.
							self
								ppOn: aStream
								tail: list tail
								position: position]
						ifFalse: 
							[self ppOn: aStream space: 1.
							self
								ppOn: aStream
								tail: list tail
								position: position + 1]]
				ifFalse: [(list tail isKindOf: LispList)
						ifFalse: 
							[aStream nextPutAll: ' . '.
							self ppOn: aStream object: list tail]].
			aStream nextPutAll: ')']! !

!LispCons methodsFor: 'pretty printing' stamp: ''!
ppOn: aStream object: anObject 
	(anObject isKindOf: Symbol)
		ifTrue: [^aStream nextPutAll: anObject asString].
	(anObject isKindOf: String)
		ifTrue: 
			[aStream nextPutAll: '"'.
			anObject
				do: 
					[:char | 
					char = $" ifTrue: [aStream nextPut: $"].
					aStream nextPut: char].
			^aStream nextPutAll: '"'].
	(anObject isKindOf: Number)
		ifTrue: [^anObject storeOn: aStream].
	(anObject isMemberOf: LispNil)
		ifTrue: [^aStream nextPutAll: 'nil'].
	aStream nextPutAll: '{'.
	aStream nextPutAll: (anObject printString contractTo: 80).
	aStream nextPutAll: '}'! !

!LispCons methodsFor: 'pretty printing' stamp: ''!
ppOn: aStream space: anInteger 
	anInteger timesRepeat: [aStream nextPut: Character space]! !

!LispCons methodsFor: 'pretty printing' stamp: ''!
ppOn: aStream spaceAndTab: anInteger 
	| tabs spaces |
	tabs := anInteger // self tabStop.
	spaces := anInteger \\ self tabStop.
	tabs * (self tabStop // 4) timesRepeat: [aStream tab].
	spaces timesRepeat: [aStream space]! !

!LispCons methodsFor: 'pretty printing' stamp: ''!
ppOn: aStream tail: list position: position 
	list null ifTrue: [^self].
	(list tail isKindOf: LispList)
		ifTrue: [list tail null
				ifTrue: 
					[self ppOn: aStream space: 1.
					self
						ppOn: aStream
						list: list head
						position: position + 1]
				ifFalse: 
					[self ppOn: aStream space: 1.
					self
						ppOn: aStream
						list: list head
						position: position + 1.
					aStream cr.
					self ppOn: aStream spaceAndTab: position.
					self
						ppOn: aStream
						tail: list tail
						position: position]]
		ifFalse: 
			[self ppOn: aStream space: 1.
			self
				ppOn: aStream
				list: list head
				position: position + 1.
			aStream nextPutAll: ' . '.
			self ppOn: aStream object: list tail]! !

!LispCons methodsFor: 'pretty printing' stamp: ''!
ppString
	| stream |
	stream := WriteStream on: (String new: 20).
	self ppOn: stream.
	^stream contents! !

!LispCons methodsFor: 'printing' stamp: ''!
printOn: aStream 
	self printOn: aStream level: 0! !

!LispCons methodsFor: 'printing' stamp: ''!
printOn: aStream level: level 
	| verticalLevel |
	verticalLevel := self class verticalLevel.
	(verticalLevel ~= 0 and: [level >= verticalLevel])
		ifTrue: 
			[aStream nextPutAll: '( ... )'.
			^self].
	self null ifTrue: [^super printOn: aStream].
	aStream nextPutAll: '('.
	(head isKindOf: self class)
		ifTrue: [head printOn: aStream level: level + 1]
		ifFalse: [self printOn: aStream object: head].
	(tail isKindOf: LispList)
		ifTrue: [self
				printOn: aStream
				tail: tail
				level: level]
		ifFalse: 
			[aStream nextPutAll: ' . '.
			self printOn: aStream object: tail.
			^aStream nextPutAll: ')']! !

!LispCons methodsFor: 'printing' stamp: ''!
printOn: aStream object: anObject 
	(anObject isKindOf: Symbol)
		ifTrue: [^aStream nextPutAll: anObject asString].
	(anObject isKindOf: String)
		ifTrue: 
			[aStream nextPutAll: '"'.
			anObject
				do: 
					[:char | 
					char = $" ifTrue: [aStream nextPut: $"].
					aStream nextPut: char].
			^aStream nextPutAll: '"'].
	(anObject isKindOf: Number)
		ifTrue: [^anObject storeOn: aStream].
	(anObject isMemberOf: LispNil)
		ifTrue: [^aStream nextPutAll: 'nil'].
	aStream nextPutAll: '{'.
	aStream nextPutAll: (anObject printString contractTo: 80).
	aStream nextPutAll: '}'! !

!LispCons methodsFor: 'printing' stamp: ''!
printOn: aStream tail: cdr level: level 
	| tailPart count horizontalLevel |
	cdr null ifTrue: [^aStream nextPutAll: ')'].
	tailPart := cdr.
	count := 1.
	horizontalLevel := self class horizontalLevel.
	tailPart do: 
		[:each | 
		(horizontalLevel ~= 0 and: [count >= horizontalLevel])
			ifTrue: 
				[aStream nextPutAll: ' ... )'.
				^self].
		aStream nextPutAll: ' '.
		(each isKindOf: self class)
			ifTrue: [tailPart head printOn: aStream level: level + 1]
			ifFalse: [self printOn: aStream object: each].
		tailPart := tailPart tail.
		count := count + 1].
	(tailPart isKindOf: LispList)
		ifTrue: [aStream nextPutAll: ')']
		ifFalse: 
			[aStream nextPutAll: ' . '.
			self printOn: aStream object: tailPart.
			aStream nextPutAll: ')']! !

!LispCons methodsFor: 'functions' stamp: ''!
reverse
	| list |
	list := self class null.
	self do: [:each | list := self class head: each tail: list].
	^list! !

!LispCons methodsFor: 'functions' stamp: ''!
rplaca: anObject 
	self head: anObject! !

!LispCons methodsFor: 'functions' stamp: ''!
rplacd: anObject 
	self tail: anObject! !

!LispCons methodsFor: 'saving' stamp: ''!
saveOn: aStream 
	self
		saveOn: aStream
		list: self
		position: 0.
	aStream cr! !

!LispCons methodsFor: 'saving' stamp: ''!
saveOn: aStream list: list position: position 
	| location length |
	(list isKindOf: self class)
		ifFalse: [^self saveOn: aStream object: list].
	(list head isKindOf: self class) not
		ifTrue: 
			[aStream nextPutAll: '('.
			location := aStream position.
			self saveOn: aStream object: list head.
			(list tail isKindOf: LispList)
				ifTrue: 
					[length := aStream position - location min: 40.
					length := 0.
					self
						saveOn: aStream
						tail: list tail
						position: position + 1 + length]
				ifFalse: 
					[aStream nextPutAll: ' . '.
					self saveOn: aStream object: list tail].
			aStream nextPutAll: ')']
		ifFalse: 
			[aStream nextPutAll: '('.
			self
				saveOn: aStream
				list: list head
				position: position + 1.
			(list tail isKindOf: self class)
				ifTrue: [(list tail head isKindOf: self class)
						ifTrue: 
							[aStream cr.
							self saveOn: aStream spaceAndTab: position.
							self
								saveOn: aStream
								tail: list tail
								position: position]
						ifFalse: 
							[self saveOn: aStream space: 1.
							self
								saveOn: aStream
								tail: list tail
								position: position + 1]]
				ifFalse: [(list tail isKindOf: LispList)
						ifFalse: 
							[aStream nextPutAll: ' . '.
							self saveOn: aStream object: list tail]].
			aStream nextPutAll: ')']! !

!LispCons methodsFor: 'saving' stamp: ''!
saveOn: aStream object: anObject 
	| string |
	(anObject isKindOf: Symbol)
		ifTrue: [^aStream nextPutAll: anObject asString].
	(anObject isKindOf: String)
		ifTrue: 
			[aStream nextPutAll: '"'.
			anObject
				do: 
					[:char | 
					char = $" ifTrue: [aStream nextPut: $"].
					aStream nextPut: char].
			^aStream nextPutAll: '"'].
	(anObject isKindOf: Integer)
		ifTrue: [^anObject storeOn: aStream].
	(anObject isKindOf: Float)
		ifTrue: [^anObject storeOn: aStream].
	"(anObject isKindOf: Double)
		ifTrue: [^anObject storeOn: aStream]."
	(anObject isMemberOf: LispNil)
		ifTrue: [^aStream nextPutAll: 'nil'].
	aStream nextPutAll: '{'.
	((anObject isKindOf: Point)
		or: [anObject isKindOf: Rectangle])
		ifTrue: [string := anObject printString]
		ifFalse: [string := anObject storeString].
	aStream nextPutAll: string.
	aStream nextPutAll: '}'! !

!LispCons methodsFor: 'saving' stamp: ''!
saveOn: aStream space: anInteger 
	anInteger timesRepeat: [aStream nextPut: Character space]! !

!LispCons methodsFor: 'saving' stamp: ''!
saveOn: aStream spaceAndTab: anInteger 
	| tabs spaces |
	tabs := anInteger // self tabStop.
	spaces := anInteger \\ self tabStop.
	tabs timesRepeat: [aStream tab].
	spaces timesRepeat: [aStream space]! !

!LispCons methodsFor: 'saving' stamp: ''!
saveOn: aStream tail: list position: position 
	list null ifTrue: [^self].
	(list tail isKindOf: LispList)
		ifTrue: [list tail null
				ifTrue: 
					[self saveOn: aStream space: 1.
					self
						saveOn: aStream
						list: list head
						position: position + 1]
				ifFalse: 
					[self saveOn: aStream space: 1.
					self
						saveOn: aStream
						list: list head
						position: position + 1.
					aStream cr.
					self saveOn: aStream spaceAndTab: position.
					self
						saveOn: aStream
						tail: list tail
						position: position]]
		ifFalse: 
			[self saveOn: aStream space: 1.
			self
				saveOn: aStream
				list: list head
				position: position + 1.
			aStream nextPutAll: ' . '.
			self saveOn: aStream object: list tail]! !

!LispCons methodsFor: 'saving' stamp: ''!
saveString
	| stream |
	stream := WriteStream on: (String new: 20).
	self saveOn: stream.
	^stream contents! !

!LispCons methodsFor: 'private' stamp: ''!
tabStop
	^8! !

!LispCons methodsFor: 'accessing' stamp: ''!
tail
	^tail! !

!LispCons methodsFor: 'accessing' stamp: ''!
tail: anObject 
	^tail := anObject! !

!LispList class methodsFor: 'instance creation' stamp: ''!
cell
	^self subclassResponsibility! !

!LispList class methodsFor: 'copyright' stamp: ''!
copyright
	^'Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.'! !

!LispList class methodsFor: 'instance creation' stamp: ''!
head: headObject 
	^self subclassResponsibility! !

!LispList class methodsFor: 'instance creation' stamp: ''!
head: headObject tail: tailObject 
	^self subclassResponsibility! !

!LispList class methodsFor: 'instance creation' stamp: ''!
list: anArray 
	"LispCons list: #(1 2 3 4)"

	| size list |
	size := anArray size.
	list := self null.
	size
		to: 1
		by: -1
		do: [:i | list := self head: (anArray at: i)
						tail: list].
	^list! !

!LispList class methodsFor: 'instance creation' stamp: ''!
new: anInteger 
	"LispCons new: 5"

	| newList |
	newList := self null.
	anInteger timesRepeat: [newList := self head: self null tail: newList].
	^newList! !

!LispList class methodsFor: 'instance creation' stamp: ''!
null
	^self subclassResponsibility! !

!LispList class methodsFor: 'copyright' stamp: ''!
system
	^'Goodies'! !

!LispList class methodsFor: 'copyright' stamp: ''!
version
	^'003'! !

!LispList class methodsFor: 'instance creation' stamp: ''!
with: anObject 
	"LispCons with: 1"

	^self head: anObject! !

!LispList class methodsFor: 'instance creation' stamp: ''!
with: firstObject with: secondObject 
	"LispCons with: 1 with: 2"

	^self head: firstObject tail: (self with: secondObject)! !

!LispList class methodsFor: 'instance creation' stamp: ''!
with: firstObject with: secondObject with: thirdObject 
	"LispCons with: 1 with: 2 with: 3"

	^self head: firstObject tail: (self with: secondObject with: thirdObject)! !

!LispList class methodsFor: 'instance creation' stamp: ''!
with: firstObject with: secondObject with: thirdObject with: fourthObject 
	"LispCons with: 1 with: 2 with: 3 with: 4"

	^self head: firstObject tail: (self
			with: secondObject
			with: thirdObject
			with: fourthObject)! !

!LispList methodsFor: 'accessing' stamp: ''!
at: indexInteger put: anObject 
	^self subscriptOutOfBoundsError: indexInteger! !

!LispList methodsFor: 'error handling' stamp: 'nishis 6/28/1998 06:02'!
error: labelString with: contentsString
	| handler |
	(handler _ Processor activeProcess errorHandler) notNil
		ifTrue:
			[handler value: labelString value: self]
		ifFalse: 
			[Debugger
				openContext: thisContext
				label: labelString
				contents: contentsString]! !

!LispList methodsFor: 'testing' stamp: ''!
isCons
	^self null not! !

!LispList methodsFor: 'testing' stamp: ''!
null
	^false! !

!LispList methodsFor: 'accessing' stamp: ''!
size
	| tally |
	tally := 0.
	self do: [:each | tally := tally + 1].
	^tally! !

!LispList methodsFor: 'private' stamp: 'nishis 4/25/98 04:06'!
subscriptOutOfBoundsError: index 
	^self error: 'subscript out of bounds error' with: 'subscript out of bounds: ' , index printString! !

LispList subclass: #LispNil
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lisp-Interpreter'!

LispNil class
	instanceVariableNames: 'null'!

LispNil class
	instanceVariableNames: 'null'!

!LispNil class methodsFor: 'instance creation' stamp: ''!
cell
	^LispCons cell! !

!LispNil class methodsFor: 'copyright' stamp: ''!
copyright
	^'Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.'! !

!LispNil class methodsFor: 'instance creation' stamp: ''!
head: headObject 
	^self shouldNotImplement! !

!LispNil class methodsFor: 'instance creation' stamp: ''!
head: headObject tail: tailObject 
	^self shouldNotImplement! !

!LispNil class methodsFor: 'class initialization' stamp: ''!
initialize
	"LispNil initialize."

	self null! !

!LispNil class methodsFor: 'instance creation' stamp: ''!
new
	^self null! !

!LispNil class methodsFor: 'instance creation' stamp: ''!
null
	null isNil ifTrue: [null := super new].
	^null! !

!LispNil class methodsFor: 'copyright' stamp: ''!
system
	^'Goodies'! !

!LispNil class methodsFor: 'copyright' stamp: ''!
version
	^'003'! !

!LispNil methodsFor: 'adding' stamp: ''!
add: newObject 
	^self shouldNotImplement! !

!LispNil methodsFor: 'functions' stamp: ''!
append: list 
	^list! !

!LispNil methodsFor: 'enumerating' stamp: ''!
do: aBlock 
	^self! !

!LispNil methodsFor: 'accessing' stamp: ''!
head
	^self! !

!LispNil methodsFor: 'functions' stamp: ''!
length
	^0! !

!LispNil methodsFor: 'functions' stamp: ''!
mapcdr: aBlock 
	^self! !

!LispNil methodsFor: 'functions' stamp: ''!
member: anObject 
	^self! !

!LispNil methodsFor: 'functions' stamp: ''!
nconc: list 
	^list! !

!LispNil methodsFor: 'functions' stamp: ''!
nth: nth 
	^self! !

!LispNil methodsFor: 'testing' stamp: ''!
null
	^true! !

!LispNil methodsFor: 'pretty printing' stamp: ''!
ppOn: aStream 
	aStream nextPutAll: 'nil'.
	aStream cr! !

!LispNil methodsFor: 'pretty printing' stamp: ''!
ppString
	^'nil\' withCRs! !

!LispNil methodsFor: 'printing' stamp: ''!
printOn: aStream 
	aStream nextPutAll: 'nil'! !

!LispNil methodsFor: 'functions' stamp: ''!
reverse
	^self! !

!LispNil methodsFor: 'saving' stamp: ''!
saveOn: aStream 
	aStream nextPutAll: 'nil'! !

!LispNil methodsFor: 'saving' stamp: ''!
saveString
	^'nil'! !

!LispNil methodsFor: 'accessing' stamp: ''!
tail
	^self! !

Object subclass: #LispScanner
	instanceVariableNames: 'source mark token tokenType typeTable failBlock compiler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lisp-Interpreter'!

LispScanner subclass: #LispParser
	instanceVariableNames: 'prevMark prevToken prevTokenType'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lisp-Interpreter'!

!LispParser class methodsFor: 'copyright' stamp: ''!
copyright
	^'Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.'! !

!LispParser class methodsFor: 'examples' stamp: ''!
example1
	"LispParser example1."

	| list |
	list := LispParser parse: '(1 2 3 4 (5 6 7 8 9) 10)'.
	^list! !

!LispParser class methodsFor: 'examples' stamp: ''!
example2
	"LispParser example2."

	| list |
	list := LispParser parse: '
			(10 (1 2)
			 20 (3 4 . 100)
			 30 (5 6) . 200)
	'.
	^list! !

!LispParser class methodsFor: 'examples' stamp: ''!
example3
	"LispParser example3."

	| list |
	list := LispParser parse: '
			(PetriNet Aoki
				(Place
					(p1 . {100@100})
					(p2 . {200@200})
					(p3 . {300@300})
					(p4 . {400@400})
					(p5 . {500@500}))
				(Transition
					(t1 . {100@100})
					(t2 . {200@200})
					(t3 . {300@300})
					(t4 . {400@400})
					(t5 . {500@500}))
				(InputFunction
					(t1 p1 p2 p3 p4 p5)
					(t2 . p4)
					(t3 . p5))
				(OutputFunction
					(t1 p1 p2 p3 p4 p5)
					(t2 . p4)
					(t3 . p5))
				(Marking {#(1 2 3 4 5)}))'.
	Transcript cr; show: list saveString.
	list := LispParser parse: list saveString.
	^list! !

!LispParser class methodsFor: 'examples' stamp: ''!
example4
	"LispParser example4."

	| list |
	list := LispParser parse: '(1 2 3 4 (5 6 7 ~ 8 9) 10)'.
	^list! !

!LispParser class methodsFor: 'examples' stamp: ''!
example5
	"LispParser example5."

	| list |
	list := LispParser parse: '(1 2 3 4 (5 6 7 {100@100 8 9) 10)'.	"error"
	^list! !

!LispParser class methodsFor: 'examples' stamp: ''!
example6
	"LispParser example6."

	| list |
	list := LispParser parse: '(1 2 3 4 (5 6 7 ''aaaaa 8 9) 10)'.	"error"
	^list! !

!LispParser class methodsFor: 'examples' stamp: ''!
example7
	"LispParser example7."

	| list |
	list := LispParser parse: ' `(`(1 2 `3) . `4) '.	"`(`(1 2 `3) . `4) -> (quote ((quote (1 2 (quote 3))) quote 4))"
	^list! !

!LispParser class methodsFor: 'private' stamp: ''!
makeStream: aStream 
	| stream |
	((aStream isKindOf: String)
		or: [aStream isKindOf: Text])
		ifTrue: [stream := ReadStream on: aStream asString]
		ifFalse: [stream := aStream].
	^stream! !

!LispParser class methodsFor: 'utilities' stamp: ''!
parse: aStream 
	^self new parse: (self makeStream: aStream)! !

!LispParser class methodsFor: 'utilities' stamp: ''!
parse: aStream ifFail: aBlock 
	^self new parse: (self makeStream: aStream)
		ifFail: aBlock! !

!LispParser class methodsFor: 'copyright' stamp: ''!
system
	^'Goodies'! !

!LispParser class methodsFor: 'copyright' stamp: ''!
version
	^'003'! !

!LispParser methodsFor: 'private' stamp: ''!
init: sourceStream ifFail: aBlock 
	super on: sourceStream.
	failBlock := aBlock! !

!LispParser methodsFor: 'scanning' stamp: ''!
nextToken
	prevMark := mark.
	prevToken := token.
	prevTokenType := tokenType.
	^super nextToken! !

!LispParser methodsFor: 'parsing' stamp: 'nishis 4/25/98 04:05'!
parse: sourceStream 
	| label string |
	^self parse: sourceStream
		ifFail: 
			[:errorMessage | 
			label := errorMessage , ' near ' , (token printString contractTo: 20).
			string := source upToEnd.
			string isEmpty
				ifTrue: [string := '--> end of file']
				ifFalse: [string := '--> ' , (string contractTo: 30)].
			self error: 'parse error.' with: label, '\' withCRs , string.
			^LispNil null]! !

!LispParser methodsFor: 'parsing' stamp: ''!
parse: sourceStream ifFail: aBlock 
	| result |
	self init: sourceStream ifFail: aBlock.
	result := self scan.
	^result! !

!LispParser methodsFor: 'scan and parse' stamp: ''!
scan
	source atEnd ifTrue: [^LispCons null].
	^self scanList! !

!LispParser methodsFor: 'scan and parse' stamp: ''!
scanList
	| expression |
	self nextToken.
	tokenType == #eof ifTrue: [^LispCons null].
	tokenType == #number ifTrue: [^token].
	tokenType == #string ifTrue: [^token].
	tokenType == #object ifTrue: [^token].
	tokenType == #symbol ifTrue: [^token].
	tokenType == #quote
		ifTrue: 
			[expression := LispCons head: self scanList tail: LispCons null.
			^LispCons head: #quote tail: expression].
	tokenType == #leftParenthesis ifTrue: [^self scanListAux].
	^failBlock value: 'Syntax error'! !

!LispParser methodsFor: 'scan and parse' stamp: ''!
scanListAux
	| cdr |
	self nextToken.
	tokenType == #eof ifTrue: [^LispCons null].
	tokenType == #rightParenthesis ifTrue: [^LispCons null].
	tokenType == #leftParenthesis ifTrue: [^LispCons head: self scanListAux tail: self scanListAux].
	tokenType == #number ifTrue: [^LispCons head: token tail: self scanListAux].
	tokenType == #string ifTrue: [^LispCons head: token tail: self scanListAux].
	tokenType == #object ifTrue: [^LispCons head: token tail: self scanListAux].
	tokenType == #symbol ifTrue: [^LispCons head: token tail: self scanListAux].
	tokenType == #period
		ifTrue: 
			[cdr := self scanList.
			self nextToken.
			tokenType == #rightParenthesis
				ifTrue: [^cdr]
				ifFalse: [^failBlock value: 'Syntax error']].
	tokenType == #quote
		ifTrue: 
			[cdr := LispCons head: self scanList tail: LispCons null.
			cdr := LispCons head: #quote tail: cdr.
			^LispCons head: cdr tail: self scanListAux].
	self unNextToken.
	^failBlock value: 'Syntax error'! !

!LispParser methodsFor: 'scanning' stamp: ''!
unNextToken
	super unNextToken.
	mark := prevMark.
	token := prevToken.
	tokenType := prevTokenType! !

!LispScanner class methodsFor: 'copyright' stamp: ''!
copyright
	^'Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.'! !

!LispScanner class methodsFor: 'instance creation' stamp: ''!
new
	^(super new) initScanner; yourself! !

!LispScanner class methodsFor: 'copyright' stamp: ''!
system
	^'Goodies'! !

!LispScanner class methodsFor: 'copyright' stamp: ''!
version
	^'003'! !

!LispScanner methodsFor: 'error handling' stamp: 'nishis 6/28/1998 06:02'!
error: labelString with: contentsString
	| handler |
	(handler _ Processor activeProcess errorHandler) notNil
		ifTrue:
			[handler value: labelString value: self]
		ifFalse: 
			[Debugger
				openContext: thisContext
				label: labelString
				contents: contentsString]! !

!LispScanner methodsFor: 'initialize-release' stamp: 'nishis 4/25/98 04:05'!
initScanner
	| label string |
	typeTable := LispScannerTable scannerTable.
	failBlock := 
			[:errorMessage | 
			label := errorMessage , ' near ' , (token printString contractTo: 10).
			string := source upToEnd.
			string isEmpty
				ifTrue: [string := '--> end of file']
				ifFalse: [string := '--> ' , (string contractTo: 30)].
			self error: 'scan error.' with: label, '\' withCRs , string].
	Compiler isNil
		ifTrue: ["when I am a run time image"
			compiler := LispSmallCompiler new]
		ifFalse: [compiler := Compiler new]! !

!LispScanner methodsFor: 'scanning' stamp: ''!
multiChar: type 
	self perform: type! !

!LispScanner methodsFor: 'private' stamp: ''!
nextChar
	| char |
	char := source next.
	char = Character linefeed ifTrue: [char := Character cr].
	char = Character cr
		ifTrue: 
			[source peek = Character linefeed ifTrue: [source next].
			char := Character cr].
	char isNil ifTrue: [char := $!!].
	^char! !

!LispScanner methodsFor: 'scanning' stamp: ''!
nextToken
	| char |
	mark := source position.
	char := self peekChar.
	tokenType := typeTable at: char asInteger.
	[tokenType == #xDelimiter]
		whileTrue: 
			[self nextChar.
			char := self peekChar.
			tokenType := typeTable at: char asInteger].
	(tokenType at: 1)
		= $x
		ifTrue: [self multiChar: tokenType]
		ifFalse: [self singleChar: tokenType].
	^token! !

!LispScanner methodsFor: 'reading' stamp: ''!
numberFrom: aStream 
	^Number readFrom: aStream! !

!LispScanner methodsFor: 'reading' stamp: ''!
objectFrom: aStream 
	| buffer char |
	buffer := WriteStream on: (String new: 20).
	char := aStream next.
	
	[char := aStream next.
	char = $}]
		whileFalse: 
			[char == nil ifTrue: [^failBlock value: 'Syntax error unmatched ${'].
			buffer nextPut: char].
	^compiler
		evaluate: buffer contents
		in: nil
		to: nil
		notifying: nil
		ifFail: []! !

!LispScanner methodsFor: 'initialize-release' stamp: ''!
on: inputStream 
	source := inputStream.
	mark := source position! !

!LispScanner methodsFor: 'private' stamp: ''!
peekChar
	| char |
	char := source peek.
	char = Character linefeed ifTrue: [char := Character cr].
	char = Character cr ifTrue: [char := Character cr].
	char isNil ifTrue: [char := $!!].
	^char! !

!LispScanner methodsFor: 'scanning' stamp: ''!
singleChar: type 
	self nextChar.
	token := type! !

!LispScanner methodsFor: 'reading' stamp: ''!
stringFrom: aStream 
	| buffer char string |
	buffer := WriteStream on: (String new: 20).
	char := aStream next.
	char = $" ifTrue: [
		[char := aStream peek.
		char ~~ nil]
			whileTrue: 
				[char = $"
					ifTrue: 
						[aStream next.
						char := aStream peek.
						char = $" ifFalse: [^String fromString: buffer contents]].
				buffer nextPut: aStream next]].
	string := aStream upToEnd.
	string size > 100 ifTrue: [string := string copyFrom: 1 to: 100].
	^failBlock value: 'Syntax error unmatched $'''! !

!LispScanner methodsFor: 'reading' stamp: ''!
symbolFrom: aStream 
	| buffer char type |
	buffer := WriteStream on: (String new: 20).
	char := aStream peek.
	[char notNil and: [(type := typeTable at: char asInteger) == #xSymbol or: [type == #xDigit or: [type == #xSign]]]]
		whileTrue: 
			[buffer nextPut: aStream next.
			char := aStream peek].
	buffer contents = 'nil' ifTrue: [^LispNil null].
	^Symbol intern: buffer contents! !

!LispScanner methodsFor: 'private' stamp: ''!
unNextChar
	source skip: -1! !

!LispScanner methodsFor: 'scanning' stamp: ''!
unNextToken
	source position: mark! !

!LispScanner methodsFor: 'x' stamp: ''!
xBinary
	^failBlock value: 'Syntax error ' , source peek printString! !

!LispScanner methodsFor: 'x' stamp: ''!
xBrace
	tokenType := #object.
	token := self objectFrom: source! !

!LispScanner methodsFor: 'x' stamp: ''!
xComment
	| char |
	[(char := self nextChar) = Character cr]
		whileFalse: [char == nil ifTrue: [^self nextToken]].
	^self nextToken! !

!LispScanner methodsFor: 'x' stamp: ''!
xDigit
	tokenType := #number.
	token := self numberFrom: source! !

!LispScanner methodsFor: 'x' stamp: ''!
xDoubleQuote
	tokenType := #string.
	token := self stringFrom: source! !

!LispScanner methodsFor: 'x' stamp: ''!
xSign
	| char sign |
	sign := self nextChar.
	char := self peekChar.
	char isDigit
		ifTrue: 
			[tokenType := #number.
			token := self numberFrom: source.
			sign == $- ifTrue: [token := token negated]]
		ifFalse: 
			[self unNextChar.
			tokenType := #symbol.
			token := self symbolFrom: source]! !

!LispScanner methodsFor: 'x' stamp: ''!
xSymbol
	tokenType := #symbol.
	token := self symbolFrom: source! !

Object variableSubclass: #LispSmallCompiler
	instanceVariableNames: 'source mark prevEnd hereChar token tokenType saveComments currentComment buffer typeTable parseNode'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lisp-Interpreter'!

!LispSmallCompiler class methodsFor: 'copyright' stamp: ''!
copyright
	^'Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.'! !

!LispSmallCompiler class methodsFor: 'evaluating' stamp: ''!
evaluate: textOrString 
	^self
		evaluate: textOrString
		for: nil
		logged: false! !

!LispSmallCompiler class methodsFor: 'evaluating' stamp: ''!
evaluate: textOrString for: anObject logged: logFlag 
	^self
		evaluate: textOrString
		for: anObject
		notifying: nil
		logged: logFlag! !

!LispSmallCompiler class methodsFor: 'evaluating' stamp: ''!
evaluate: textOrString for: anObject notifying: aController logged: logFlag 
	^self new
		evaluate: textOrString
		in: nil
		receiver: anObject
		notifying: aController
		ifFail: [^nil]! !

!LispSmallCompiler class methodsFor: 'evaluating' stamp: ''!
evaluate: textOrString logged: logFlag 
	^self
		evaluate: textOrString
		for: nil
		logged: logFlag! !

!LispSmallCompiler class methodsFor: 'evaluating' stamp: ''!
evaluate: textOrString notifying: aController logged: logFlag 
	^self
		evaluate: textOrString
		for: nil
		notifying: aController
		logged: logFlag! !

!LispSmallCompiler class methodsFor: 'examples' stamp: ''!
example0
	"LispSmallCompiler example0."

	| stream |
	stream := WriteStream on: Array new.
	stream nextPut: (LispSmallCompiler evaluate: '(nil)').
	stream nextPut: (LispSmallCompiler evaluate: '(true)').
	stream nextPut: (LispSmallCompiler evaluate: '(false)').
	stream contents do: [:each | Transcript cr; show: each printString].
	^stream contents! !

!LispSmallCompiler class methodsFor: 'examples' stamp: 'nishis 4/25/98 04:21'!
example1
	"LispSmallCompiler example1."

	| stream |
	stream := WriteStream on: Array new.
	stream nextPut: (LispSmallCompiler evaluate: '(123)').
	stream nextPut: (LispSmallCompiler evaluate: '(123.456)').
	stream nextPut: (LispSmallCompiler evaluate: '(123.456e7)').
	stream nextPut: (LispSmallCompiler evaluate: '(-123)').
	stream nextPut: (LispSmallCompiler evaluate: '(-123.456)').
	stream nextPut: (LispSmallCompiler evaluate: '(-123.456e7)').
	stream nextPut: (LispSmallCompiler evaluate: '(-123.456e-7)').
	stream contents do: [:each | Transcript cr; show: each printString].
	^stream contents! !

!LispSmallCompiler class methodsFor: 'examples' stamp: ''!
example2
	"LispSmallCompiler example2."

	| stream |
	stream := WriteStream on: Array new.
	stream nextPut: (LispSmallCompiler evaluate: '($c)').
	stream nextPut: (LispSmallCompiler evaluate: '(''string'')').
	stream nextPut: (LispSmallCompiler evaluate: '(#symbol)').
	stream contents do: [:each | Transcript cr; show: each printString].
	^stream contents! !

!LispSmallCompiler class methodsFor: 'examples' stamp: ''!
example3
	"LispSmallCompiler example3."

	| stream |
	stream := WriteStream on: Array new.
	stream nextPut: (LispSmallCompiler evaluate: '(123 @ 456)').
	stream nextPut: (LispSmallCompiler evaluate: '(123 , 456)').
	stream nextPut: (LispSmallCompiler evaluate: '(123 , 456 , 789)').
	stream nextPut: (LispSmallCompiler evaluate: '(Point x: 123 y: 456)').
	(Smalltalk includesKey: #Jun2dPoint) 
		ifTrue: 
			[stream nextPut: (LispSmallCompiler evaluate: '(Jun2dPoint x: 123 y: 456)')].
	(Smalltalk includesKey: #Jun3dPoint) 
		ifTrue: 
			[stream nextPut: (LispSmallCompiler evaluate: '(Jun3dPoint x: 123 y: 456 z: 789)')].
	stream contents do: [:each | Transcript cr; show: each printString].
	^stream contents! !

!LispSmallCompiler class methodsFor: 'examples' stamp: 'nishis 4/25/98 04:39'!
example4
	"LispSmallCompiler example4."

	| stream |
	stream := WriteStream on: Array new.
	stream nextPut: (LispSmallCompiler evaluate: '(Color h: 0.57 s: 0.46 v: 1.0)').
	stream nextPut: (LispSmallCompiler evaluate: '(Color r: 0.57 g: 0.46 b: 1.0)').
	stream nextPut: (LispSmallCompiler evaluate: '(Color gray: 0.7)').
	stream contents do: [:each | Transcript cr; show: each printString].
	^stream contents! !

!LispSmallCompiler class methodsFor: 'examples' stamp: 'nishis 4/25/98 04:45'!
example5
	"LispSmallCompiler example5."
	"LispSmallCompiler example5 bitEdit."

	^LispSmallCompiler evaluate: '(Form extent: 6@7 bitsPerPixel: 32  fromArray: #(127 0 0 127 0 0 127 0 0 127 0 0 127 0 0 255 255 255 0 0 255 255 255 255 255 255 127 0 0 127 0 0 127 0 0 255 255 255 0 0 127 0 0 127 0 0 255 255 255 127 0 0 127 0 0 255 255 255 0 0 127 0 0 127 0 0 255 255 255 127 0 0 127 0 0 255 255 255 0 0 127 0 0 127 0 0 255 255 255 127 0 0 127 0 0 255 255 255 0 0 127 0 0 127 0 0 255 255 255 127 0 0 127 0 0 127 0 0 0 0 127 0 0 127 0 0 127 0 0 127 0 0 127 0 0 127 0 0 0 0))'! !

!LispSmallCompiler class methodsFor: 'examples' stamp: 'nishis 4/25/98 04:47'!
example6
	"LispSmallCompiler example6 bitEdit."
	"LispSmallCompiler example6 bitEdit."

	^LispSmallCompiler evaluate: '(Form extent: 14@15 bitsPerPixel: 16  fromArray: ( ''????????????????????????????@@@@????????????@@@@????????@@@@@@C???????????<@@@C???<@@@@@@@@@@@C???<@@@@@@@@@@@C???<@@O???0@@@O???????0@@@O???????0@@@O???????????0@@@O???????0@@????@@@@????????@@@@????????@@@@????????????@@@@????????@@C???<@@@C???????<@@@C???????<@@@C???????????<@@@C???????<@@O???0@@@O???????0@@@O???????0@@@O???????????????0@@@O???0@@????????????????????????????????????????????????????????@@C???????????????????????????????????????????????????????<@@O???????????????????????????????????????????????????????0@@????????????????????????????????????????????????????????@@@@@@@@@@@@@@C???????<@@@@@@@@@@@@@@@@@@@C???<@@@@@@@@@@@@@@O???????????0@@@O???0@@@O???????????????????0@@@O???????0@@????????????@@@@????@@@@@@@@@@@@@@@@????????@@@@@@@@@@@@@@C???????????<@@@C???<@@@C???????????<@@@C???<@@@C???????<@@O???0@@@@@@@O???????????????????????0@@@O???????????????0@@'' asByteArray))'! !

!LispSmallCompiler class methodsFor: 'examples' stamp: 'nishis 4/25/98 04:49'!
example7
	"LispSmallCompiler example7."
	"LispSmallCompiler example7 bitEdit."

	^LispSmallCompiler evaluate: '(Form extent: 16@16 bitsPerPixel: 1 fromArray: #(192 16 0 0 224 56 0 0 240 56 0 0 249 255 0 0 255 255 0 0 255 255 0 0 255 254 0 0 255 254 0 0 254 254 0 0 255 239 0 0 255 199 0 0 143 199 0 0 15 0 0 0 7 128 0 0 7 128 0 0 3 0 0 0); yourself)'! !

!LispSmallCompiler class methodsFor: 'examples' stamp: 'nishis 4/25/98 04:50'!
example8
	"LispSmallCompiler example8."
	"LispSmallCompiler example8 edit."

	^LispSmallCompiler evaluate: '(Form extent: 256@64 bitsPerPixel: 8 fromArray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a'' asByteArray))'! !

!LispSmallCompiler class methodsFor: 'instance creation' stamp: ''!
new
	^(super new) initScanner; yourself! !

!LispSmallCompiler class methodsFor: 'copyright' stamp: ''!
system
	^'Goodies'! !

!LispSmallCompiler class methodsFor: 'copyright' stamp: ''!
version
	^'003'! !

!LispSmallCompiler methodsFor: 'parsing' stamp: ''!
arrayLiteral
	| stream |
	stream := WriteStream on: (Array new: 16).
	[tokenType = #rightParenthesis]
		whileFalse: 
			[tokenType == #doIt ifTrue: [^self notify: 'array element or right parenthesis'].
			self arrayLiteralElement.
			stream nextPut: parseNode].
	parseNode := stream contents.
	self scanToken! !

!LispSmallCompiler methodsFor: 'parsing' stamp: ''!
arrayLiteralElement
	self constant ifTrue: [^self].
	tokenType = #leftParenthesis
		ifTrue: 
			[self scanToken.
			self arrayLiteral.
			^self].
	tokenType = #leftBracket
		ifTrue: 
			[self scanToken.
			self byteArrayLiteral.
			^self].
	self scanLitToken.
	parseNode := token.
	self scanToken! !

!LispSmallCompiler methodsFor: 'parsing' stamp: ''!
byteArrayLiteral
	| stream |
	stream := WriteStream on: (ByteArray new: 16).
	[tokenType = #rightBracket]
		whileFalse: 
			[(tokenType == #number and: [token isInteger and: [token >= 0 and: [token <= 255]]])
				ifFalse: [^self notify: '8-bit integer or right bracket'].
			stream nextPut: token.
			self scanToken].
	parseNode := stream contents.
	self scanToken! !

!LispSmallCompiler methodsFor: 'translating' stamp: ''!
compile: aString 
	| tokenArray |
	tokenArray := self scanTokens: aString.
	^self translate: tokenArray! !

!LispSmallCompiler methodsFor: 'parsing' stamp: ''!
constant
	| size |
	tokenType == #word
		ifTrue: 
			[size := token size.
			size = 5
				ifTrue: [token = 'false'
						ifTrue: 
							[self scanToken.
							parseNode := false.
							^true]]
				ifFalse: [size = 4
						ifTrue: [token = 'true'
								ifTrue: 
									[self scanToken.
									parseNode := true.
									^true]]
						ifFalse: [size = 3 ifTrue: [token = 'nil'
									ifTrue: 
										[self scanToken.
										parseNode := nil.
										^true]]]].
			^false].
	(tokenType == #string or: [tokenType == #number or: [tokenType == #character]])
		ifTrue: 
			[parseNode := token.
			self scanToken.
			^true].
	(token == #- and: [hereChar notNil and: [hereChar isDigit]])
		ifTrue: 
			[self scanToken.
			parseNode := token negated.
			self scanToken.
			^true].
	tokenType == #literalQuote ifFalse: [^false].
	self scanToken.
	(tokenType == #word or: [tokenType == #binary or: [tokenType == #verticalBar or: [tokenType == #string]]])
		ifTrue: 
			[parseNode := token asSymbol.
			self scanToken.
			^true].
	tokenType == #keyword
		ifTrue: 
			[self scanLitKeywords.
			parseNode := token asSymbol.
			self scanToken.
			^true].
	tokenType == #leftParenthesis
		ifTrue: 
			[self scanToken.
			self arrayLiteral.
			^true].
	tokenType == #leftBracket
		ifTrue: 
			[self scanToken.
			self byteArrayLiteral.
			^true].
	^self notify: 'word, binary, keyword, (, or ['! !

!LispSmallCompiler methodsFor: 'error handling' stamp: 'nishis 6/28/1998 06:02'!
error: labelString with: contentsString
	| handler |
	(handler _ Processor activeProcess errorHandler) notNil
		ifTrue:
			[handler value: labelString value: self]
		ifFalse: 
			[Debugger
				openContext: thisContext
				label: labelString
				contents: contentsString]! !

!LispSmallCompiler methodsFor: 'public access' stamp: ''!
evaluate: textOrStream in: aContext receiver: receiver notifying: aRequestor ifFail: failBlock 
	| aString |
	(textOrStream isKindOf: PositionableStream)
		ifTrue: [aString := textOrStream contents asString]
		ifFalse: [aString := textOrStream asString].
	^self compile: aString! !

!LispSmallCompiler methodsFor: 'public access' stamp: ''!
evaluate: textOrStream in: aContext to: receiver notifying: aRequestor ifFail: failBlock 
	| value |
	value := nil.
	Cursor execute showWhile: [value := self
					evaluate: textOrStream
					in: aContext
					receiver: receiver
					notifying: aRequestor
					ifFail: failBlock].
	^value! !

!LispSmallCompiler methodsFor: 'initialize-release' stamp: ''!
initScanner
	buffer := WriteStream on: (String new: 32).
	saveComments := true.
	typeTable := LispSmallTypeTable typeTable! !

!LispSmallCompiler methodsFor: 'error handling' stamp: 'nishis 4/25/98 04:08'!
notify: string 
	self error: 'Lisp Compiler error' with: string! !

!LispSmallCompiler methodsFor: 'error handling' stamp: ''!
offEnd: aString 
	^self notify: aString! !

!LispSmallCompiler methodsFor: 'initialize-release' stamp: ''!
on: inputStream 
	source := inputStream.
	hereChar := source next.
	prevEnd := 0! !

!LispSmallCompiler methodsFor: 'initialize-release' stamp: ''!
scan: inputStream 
	self on: inputStream.
	self scanToken! !

!LispSmallCompiler methodsFor: 'public access' stamp: ''!
scanFieldNames: stringOrArray 
	"LispSmallCompiler new scanFieldNames: 'abc def ghi'."

	| stream |
	(stringOrArray isMemberOf: Array)
		ifTrue: [^stringOrArray].
	stringOrArray isEmpty ifTrue: [^Array new: 0].
	self scan: (ReadStream on: stringOrArray asString).
	stream := WriteStream on: (Array new: 10).
	[tokenType = #doIt]
		whileFalse: 
			[tokenType = #word ifTrue: [stream nextPut: token].
			self scanToken].
	^stream contents! !

!LispSmallCompiler methodsFor: 'scanning' stamp: ''!
scanLitKeywords
	| inKeywords t lastP lastCh |
	inKeywords := true.
	[inKeywords and: [hereChar notNil and: [(typeTable at: hereChar asInteger)
				= #xLetter]]]
		whileTrue: 
			[t := token.
			lastP := source position.
			lastCh := hereChar.
			self xLetter.
			tokenType = #keyword
				ifTrue: [token := t , token]
				ifFalse: 
					[inKeywords := false.
					token := t.
					hereChar := lastCh.
					source position: lastP.
					tokenType := #keyword]].
	token := token asSymbol! !

!LispSmallCompiler methodsFor: 'scanning' stamp: ''!
scanLitToken
	tokenType = #leftParenthesis
		ifTrue: 
			[self scanToken.
			self scanLitVec]
		ifFalse: [tokenType = #word
				ifTrue: [token := token asSymbol]
				ifFalse: [tokenType = #keyword
						ifTrue: [self scanLitKeywords]
						ifFalse: [(token == #- and: [hereChar notNil and: [(typeTable at: hereChar asInteger)
										= #xDigit]])
								ifTrue: 
									[self scanToken.
									token := token negated]
								ifFalse: [((token isMemberOf: Character)
										and: [tokenType ~~ #character])
										ifTrue: [token := token asSymbol]]]]]! !

!LispSmallCompiler methodsFor: 'scanning' stamp: ''!
scanLitVec
	| s |
	s := WriteStream on: (Array new: 16).
	[tokenType = #rightParenthesis or: [tokenType = #doIt]]
		whileFalse: 
			[self scanLitToken.
			s nextPut: token.
			self scanToken].
	token := s contents! !

!LispSmallCompiler methodsFor: 'public access' stamp: 'nishis 6/28/1998 05:42'!
scanPositionsFor: aSymbol inString: aString 
	"LispSmallCompiler new scanPositionsFor: #LispSmallCompiler inString: 
	'| x y | 
	LispSmallCompiler compile: x. 
	Smalltalk at: #LispSmallCompiler put: nil. 
	(LispSmallCompiler compile: x y: y). 
	LispSmallCompiler foo'"

	| positions string |
	self scan: (ReadStream on: aString).
	string := aSymbol asString.
	positions := WriteStream with: (Array new: 16).
	[tokenType = #doIt]
		whileFalse: 
			[(tokenType = #word and: [string = token])
				ifTrue: [positions nextPut: mark].
			self scanToken].
	^positions contents! !

!LispSmallCompiler methodsFor: 'scanning' stamp: ''!
scanToken
	| skipped |
	
	[hereChar == nil
		ifTrue: 
			[prevEnd := source position.
			mark := prevEnd + 1.
			tokenType := #doIt.
			^token := nil].
	skipped := 0.
	(tokenType := typeTable at: hereChar asInteger) == #xDelimiter]
		whileTrue: 
			[skipped := skipped + 1.
			hereChar := source next].
	mark := source position.
	prevEnd := mark - skipped - 1.
	tokenType == #xLetter
		ifTrue: [self xLetter]
		ifFalse: [(tokenType at: 1)
				= $x
				ifTrue: [self perform: tokenType]
				ifFalse: 
					[token := hereChar.
					hereChar := source next]].
	^token! !

!LispSmallCompiler methodsFor: 'public access' stamp: ''!
scanTokens: textOrString 
	"LispSmallCompiler new scanTokens: 
	'(LispSmallCompiler #(aaa #(bbb 123 456.7d) nil true false) 
	#[1 2 3 4 5 6 7 8 9] identifier keyword: 8r31 
	''string'' embedded.period key:word: . )'."

	| stream |
	self scan: (ReadStream on: textOrString asString).
	stream := WriteStream on: (Array new: 16).
	[tokenType = #doIt]
		whileFalse: 
			[self arrayLiteralElement.
			stream nextPut: parseNode].
	^stream contents! !

!LispSmallCompiler methodsFor: 'string and character' stamp: ''!
separate: separateCollection dividers: dividerCollection 
	"LispSmallCompiler new separate: #(a b c d e f g h) dividers: #(d f)."

	| indexCollection index intervalCollection s e result |
	indexCollection := OrderedCollection new.
	indexCollection add: 0.
	index := 1.
	separateCollection
		do: 
			[:each | 
			(dividerCollection includes: each)
				ifTrue: [indexCollection add: index].
			index := index + 1].
	indexCollection add: separateCollection size + 1.
	intervalCollection := OrderedCollection new.
	1 to: indexCollection size - 1
		do: 
			[:i | 
			s := indexCollection at: i.
			e := indexCollection at: i + 1.
			intervalCollection add: (s + 1 to: e - 1)].
	result := OrderedCollection new.
	intervalCollection do: [:interval | interval size > 0 ifTrue: [result add: (separateCollection copyFrom: interval first to: interval last)]].
	^result! !

!LispSmallCompiler methodsFor: 'string and character' stamp: ''!
separators
	"LispSmallCompiler new separators."

	| stream |
	stream := WriteStream on: (String new: 4).
	stream nextPut: Character space.
	stream nextPut: Character tab.
	stream nextPut: Character cr.
	stream nextPut: Character linefeed.
	^stream contents! !

!LispSmallCompiler methodsFor: 'scanning' stamp: ''!
step
	| c |
	c := hereChar.
	hereChar := source next.
	^c! !

!LispSmallCompiler methodsFor: 'string and character' stamp: ''!
stringMatch: string and: pattern 
	"LispSmallCompiler new stringMatch: ' Aoki' and: 'Ju*[A-B]?ki'."

	| s1 s2 |
	s1 := WriteStream on: (String new: 20).
	s1 nextPutAll: string.
	s1 nextPut: (Character value: 0).
	s2 := WriteStream on: (String new: 20).
	s2 nextPutAll: pattern.
	s2 nextPut: (Character value: 0).
	^self
		stringMatch: s1 contents
		sIndex: 1
		and: s2 contents
		pIndex: 1! !

!LispSmallCompiler methodsFor: 'string and character' stamp: ''!
stringMatch: string sIndex: sindex and: pattern pIndex: pindex 
	| scc c si pi ok lc |
	si := sindex.
	pi := pindex.
	scc := string at: si.
	si := si + 1.
	c := pattern at: pi.
	pi := pi + 1.
	c = $[
		ifTrue: 
			[ok := false.
			lc := 255.
			
			[c := pattern at: pi.
			pi := pi + 1.
			c ~= (Character value: 0)]
				whileTrue: [c = $]
						ifTrue: [ok
								ifTrue: [^self
										stringMatch: string
										sIndex: si
										and: pattern
										pIndex: pi]
								ifFalse: [^false]]
						ifFalse: [c = $-
								ifTrue: 
									[(lc <= scc and: [scc <= (pattern at: pi)])
										ifTrue: [ok := true].
									pi := pi + 1]
								ifFalse: 
									[lc := c.
									scc = c ifTrue: [ok := true]]]]].
	c = $? ifTrue: [scc ~= (Character value: 0)
			ifTrue: [^self
					stringMatch: string
					sIndex: si
					and: pattern
					pIndex: pi]
			ifFalse: [^false]].
	c = $*
		ifTrue: 
			[(pattern at: pi)
				= (Character value: 0) ifTrue: [^true].
			si := si - 1.
			[(string at: si)
				~= (Character value: 0)]
				whileTrue: 
					[(self
						stringMatch: string
						sIndex: si
						and: pattern
						pIndex: pi)
						ifTrue: [^true].
					si := si + 1].
			^false].
	c = (Character value: 0) ifTrue: [^scc = (Character value: 0)].
	c ~= scc ifTrue: [^false].
	scc ~= (Character value: 0)
		ifTrue: [^self
				stringMatch: string
				sIndex: si
				and: pattern
				pIndex: pi]
		ifFalse: [^false]! !

!LispSmallCompiler methodsFor: 'translating' stamp: ''!
translate: tokenArray 
	| value receiver selector |
	tokenArray isEmpty ifTrue: [^nil].
	(tokenArray size = 1 and: [tokenArray first isKindOf: Array])
		ifTrue: 
			[value := nil.
			tokenArray do: [:each | value := self translate: each].
			^value].
	tokenArray size = 1 ifTrue: [^tokenArray first].
	receiver := tokenArray at: 1.
	selector := tokenArray at: 2.
	((receiver isKindOf: Number)
		and: [selector = #@ or: [selector = #,]])
		ifTrue: [^self translateToPoint: tokenArray].
	(((receiver = #Point and: [selector = #x:])
		or: [receiver = #Jun3dPoint and: [selector = #x:]])
		or: [receiver = #Jun2dPoint and: [selector = #x:]])
		ifTrue: [^self translateToPoint: tokenArray].
	(receiver = #ByteArray and: [selector = #fromPackedString:])
		ifTrue: [^self translateToByteArray: tokenArray].
	"(((receiver = #Image and: [selector = #extent:])
		or: [receiver = #OpaqueImage and: [selector = #basicNew]])
		or: [receiver = #CachedImage and: [selector = #on:]])
		ifTrue: [^self translateToImage: tokenArray]."
	(receiver = #Form and: [selector = #extent:])
		ifTrue: [^self translateToForm: tokenArray].
	(receiver = #Color and: [(#(#r: #h: #gray:) includes: selector)
			or: [Color colorNames includes: selector]])
		ifTrue: [^self translateToColor: tokenArray].
	^self notify: 'unexpected error'! !

!LispSmallCompiler methodsFor: 'translating' stamp: 'nishis 4/19/98 22:09'!
translateToByteArray: tokenArray 
	| aString aSelector byteArray |
	(tokenArray isKindOf: ByteArray)
		ifTrue: [^tokenArray].
	aString := tokenArray at: 1.
	aSelector := tokenArray at: 2.
"	(aSymbol = #ByteArray and: [aSelector = #fromPackedString:])
		ifFalse: [^self notify: 'unexpected error'].
	byteArray := ByteArray fromPackedString: (tokenArray at: 3)."
	((aString isKindOf: String) and: [aSelector = #asByteArray])
		ifFalse: [^self notify: 'unexpected error'].
	byteArray := aString asByteArray.
	^byteArray! !

!LispSmallCompiler methodsFor: 'translating' stamp: ''!
translateToColor: tokenArray 
	| symbol collection stream selector arguments color |
	symbol := tokenArray at: 1.
	symbol = #Color ifFalse: [^self notify: 'unexpected error'].
	selector := tokenArray at: 2.
	(Color colorNames includes: selector)
		ifTrue: [color := Color perform: selector]
		ifFalse: 
			[collection := OrderedCollection new.
			stream := WriteStream on: (String new: 32).
			2
				to: tokenArray size
				by: 2
				do: 
					[:index | 
					stream nextPutAll: (tokenArray at: index).
					collection add: (tokenArray at: index + 1)].
			selector := stream contents asSymbol.
			arguments := collection asArray.
			color := Color perform: selector withArguments: arguments].
	^color! !

!LispSmallCompiler methodsFor: 'translating' stamp: 'nishis 4/19/98 22:35'!
translateToForm: tokenArray 
	| aSymbol aForm extentPoint bitsPerPixel anObject byteArray |
	aSymbol := tokenArray at: 1.
	aSymbol = #Form 
		ifFalse: [^self notify: 'unexpected error'].
	extentPoint := (tokenArray at: 3) @ (tokenArray at: 5).
	bitsPerPixel := tokenArray at: 7.
	anObject := tokenArray at: 9.
	((anObject isKindOf: Array) and: [((anObject at: 1) isKindOf: String) not]) 
		ifTrue: [byteArray := anObject]
		ifFalse: [byteArray := self translateToByteArray: (tokenArray at: 9)].
	aForm := Form
				extent: extentPoint 
				depth: bitsPerPixel 
				fromArray: byteArray 
				offset: nil.
	^aForm! !

!LispSmallCompiler methodsFor: 'translating' stamp: ''!
translateToImage: tokenArray 
	^self shouldNotImplement
"
	| aSymbol anImage extentPoint colorDepth bitsPerPixel anObject aPalette byteArray figureImage shapeImage |
	aSymbol := tokenArray at: 1.
	(aSymbol = #Image or: [aSymbol = #OpaqueImage or: [aSymbol = #CachedImage]])
		ifFalse: [^self notify: 'unexpected error'].
	aSymbol = #Image
		ifTrue: 
			[extentPoint := (tokenArray at: 3)
						@ (tokenArray at: 5).
			colorDepth := tokenArray at: 7.
			bitsPerPixel := tokenArray at: 9.
			anObject := tokenArray at: 11.
			(anObject isKindOf: Array)
				ifTrue: 
					[aPalette := self translateToPalette: (tokenArray at: 11).
					byteArray := self translateToByteArray: (tokenArray at: 13)]
				ifFalse: 
					[aPalette := self translateToPalette: (Array with: (tokenArray at: 11)
									with: (tokenArray at: 12)).
					byteArray := self translateToByteArray: (tokenArray at: 14)].
			anImage := Image
						extent: extentPoint
						depth: colorDepth
						bitsPerPixel: bitsPerPixel
						palette: aPalette
						usingBits: byteArray].
	aSymbol = #OpaqueImage
		ifTrue: 
			[figureImage := self translateToImage: (tokenArray at: 6).
			shapeImage := self translateToImage: (tokenArray at: 11).
			anImage := OpaqueImage figure: figureImage shape: shapeImage].
	aSymbol = #CachedImage
		ifTrue: 
			[anImage := self translate: (tokenArray at: 3).
			anImage := CachedImage on: anImage].
	^anImage"! !

!LispSmallCompiler methodsFor: 'translating' stamp: ''!
translateToPalette: tokenArray 
	^self shouldNotImplement
"
	| aSymbol aPalette selector collection stream sort array arguments index location it |
	aSymbol := tokenArray at: 1.
	(aSymbol = #CoveragePalette or: [aSymbol = #FixedPalette or: [aSymbol = #MappedPalette or: [aSymbol = #MonoMappedPalette]]])
		ifFalse: [^self notify: 'unexpected error'].
	aSymbol = #CoveragePalette
		ifTrue: 
			[selector := tokenArray at: 2.
			(CoveragePalette constantNames includes: selector)
				ifTrue: [aPalette := CoveragePalette perform: selector]
				ifFalse: [aPalette := CoveragePalette perform: selector with: (tokenArray at: 3)]].
	aSymbol = #FixedPalette
		ifTrue: 
			[collection := OrderedCollection new.
			stream := WriteStream on: (String new: 32).
			2
				to: tokenArray size
				by: 2
				do: 
					[:indexAt | 
					stream nextPutAll: (tokenArray at: indexAt).
					collection add: (tokenArray at: indexAt + 1)].
			selector := stream contents asSymbol.
			arguments := collection asArray.
			aPalette := FixedPalette perform: selector withArguments: arguments].
	(aSymbol = #MappedPalette or: [aSymbol = #MonoMappedPalette])
		ifTrue: 
			[sort := Smalltalk at: aSymbol.
			selector := tokenArray at: 2.
			(sort constantNames includes: selector)
				ifTrue: [aPalette := sort perform: selector]
				ifFalse: 
					[array := tokenArray at: 3.
					arguments := Array new: ((array at: 1)
									at: 3).
					array := array copyFrom: 2 to: array size - 2.
					index := array
								nextIndexOf: #at:
								from: 1
								to: array size.
					[index isNil]
						whileFalse: 
							[location := array at: index + 1.
							it := array at: index + 3.
							(it isKindOf: Array)
								ifFalse: [it := Array with: (array at: index + 3)
												with: (array at: index + 4)].
							arguments at: location put: (self translateToColor: it).
							index := array
										nextIndexOf: #at:
										from: index + 1
										to: array size].
					aPalette := sort perform: selector with: arguments]].
	^aPalette"! !

!LispSmallCompiler methodsFor: 'translating' stamp: ''!
translateToPoint: tokenArray 
	| receiver selector arguments point |
	receiver := tokenArray at: 1.
	selector := tokenArray at: 2.
	((receiver isKindOf: Number)
		and: [selector = #@ or: [selector = #,]])
		ifTrue: 
			[2
				to: tokenArray size
				by: 2
				do: 
					[:index | 
					selector := tokenArray at: index.
					arguments := Array with: (tokenArray at: index + 1).
					(receiver respondsTo: selector)
						ifFalse: [^tokenArray].
					point := receiver perform: selector withArguments: arguments.
					receiver := point].
			^point].
	(receiver = #Point "or: [receiver = #Jun3dPoint or: [receiver = #Jun2dPoint]]")
		ifFalse: [^self notify: 'unexpected error'].
	receiver = #Point ifTrue: [point := Point x: (tokenArray at: 3)
					y: (tokenArray at: 5)].
	"receiver = #Jun3dPoint ifTrue: [point := Jun3dPoint
					x: (tokenArray at: 3)
					y: (tokenArray at: 5)
					z: (tokenArray at: 7)].
	receiver = #Jun2dPoint ifTrue: [point := Jun2dPoint x: (tokenArray at: 3)
					y: (tokenArray at: 5)]."
	^point! !

!LispSmallCompiler methodsFor: 'x' stamp: ''!
xBinary
	| char |
	tokenType := #binary.
	char := hereChar.
	hereChar := source next.
	(hereChar notNil and: [(typeTable at: hereChar asInteger)
			= #xBinary and: [hereChar ~= $-]])
		ifTrue: [token := Symbol intern: (String with: char with: self step)]
		ifFalse: [token := Symbol internCharacter: char]! !

!LispSmallCompiler methodsFor: 'x' stamp: ''!
xDefault
	self notify: 'unknown character'! !

!LispSmallCompiler methodsFor: 'x' stamp: ''!
xDelimiter
	self scanToken! !

!LispSmallCompiler methodsFor: 'x' stamp: ''!
xDigit
	tokenType := #number.
	hereChar == nil ifFalse: [source skip: -1].
"	token := Number readSmalltalkSyntaxFrom: source."
	token := Number readFrom: source.
	hereChar := source next! !

!LispSmallCompiler methodsFor: 'x' stamp: ''!
xDollar
	token := source next.
	hereChar := source next.
	tokenType := #character! !

!LispSmallCompiler methodsFor: 'x' stamp: ''!
xDoubleQuote
	| savedEnd char |
	savedEnd := prevEnd.
	saveComments ifFalse: [[(char := source next) == $"]
			whileFalse: [char == nil ifTrue: [^self offEnd: 'unmatched comment quote']]]
		ifTrue: 
			[buffer reset.
			[(char := source next) == $"]
				whileFalse: 
					[char == nil ifTrue: [^self offEnd: 'unmatched comment quote'].
					buffer nextPut: char].
			currentComment == nil
				ifTrue: [currentComment := OrderedCollection with: buffer contents]
				ifFalse: [currentComment add: buffer contents].
			buffer position > 200 ifTrue: [buffer := WriteStream on: (String new: 40)]].
	hereChar := source next.
	self scanToken.
	prevEnd := savedEnd! !

!LispSmallCompiler methodsFor: 'x' stamp: ''!
xLetter
	| type char |
	buffer reset.
	buffer nextPut: hereChar.
	
	[char := source next.
	char notNil
		and: 
			[type := typeTable at: char asInteger.
			type == #xLetter or: [type == #xDigit]]]
		whileTrue: [buffer nextPut: char].
	type == #colon
		ifTrue: 
			[buffer nextPut: char.
			char := source next.
			tokenType := #keyword]
		ifFalse: [tokenType := #word].
	hereChar := char.
	token := buffer contents! !

!LispSmallCompiler methodsFor: 'x' stamp: ''!
xLitQuote
	| start |
	self step.
	self scanToken.
	tokenType = #leftParenthesis
		ifTrue: 
			[start := mark.
			self scanToken.
			self scanLitVec.
			tokenType == #doIt
				ifTrue: 
					[mark := start.
					self offEnd: 'unmatched parenthesis']]
		ifFalse: [tokenType = #word
				ifTrue: [token := token asSymbol]
				ifFalse: [tokenType = #keyword ifTrue: [self scanLitKeywords]]].
	tokenType := #literal! !

!LispSmallCompiler methodsFor: 'x' stamp: ''!
xSingleQuote
	| char |
	char := source next.
	buffer on: (String new: 32).
	
	[char == nil ifTrue: [^self offEnd: 'unmatched string quote'].
	char == $' and: [(char := source next) ~~ $']]
		whileFalse: 
			[buffer nextPut: char.
			char := source next].
	hereChar := char.
	token := buffer contents.
	tokenType := #string! !

Object variableSubclass: #LispTable
	instanceVariableNames: 'properties'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lisp-Interpreter'!

!LispTable class methodsFor: 'copyright' stamp: ''!
copyright
	^'Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.'! !

!LispTable class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize! !

!LispTable class methodsFor: 'copyright' stamp: ''!
system
	^'Goodies'! !

!LispTable class methodsFor: 'copyright' stamp: ''!
version
	^'003'! !

!LispTable methodsFor: 'adding' stamp: ''!
add: symbol 
	self intern: symbol! !

!LispTable methodsFor: 'accessing' stamp: ''!
at: symbol 
	^self getprop: symbol key: #apval! !

!LispTable methodsFor: 'accessing' stamp: ''!
at: symbol put: value 
	self intern: symbol.
	^self
		putprop: symbol
		key: #apval
		value: value! !

!LispTable methodsFor: 'error handling' stamp: 'nishis 6/28/1998 06:01'!
error: labelString with: contentsString
	| handler |
	(handler _ Processor activeProcess errorHandler) notNil
		ifTrue:
			[handler value: labelString value: self]
		ifFalse: 
			[Debugger
				openContext: thisContext
				label: labelString
				contents: contentsString]! !

!LispTable methodsFor: 'private' stamp: 'nishis 4/25/98 04:09'!
errorSymbolNotFound
	self error: 'symbol not found' with: 'symbol not found'! !

!LispTable methodsFor: 'property access' stamp: ''!
getprop: identifier key: key 
	| property |
	property := properties at: identifier ifAbsent: [^self errorSymbolNotFound].
	^property at: key ifAbsent: [^nil]! !

!LispTable methodsFor: 'accessing' stamp: ''!
identifiers
	^properties keys asSortedCollection! !

!LispTable methodsFor: 'initialize-release' stamp: ''!
initialize
	properties := Dictionary new! !

!LispTable methodsFor: 'private' stamp: ''!
intern: symbol 
	(properties includesKey: symbol)
		ifFalse: [properties at: symbol put: Dictionary new].
	^symbol! !

!LispTable methodsFor: 'property access' stamp: ''!
putprop: identifier key: key value: value 
	| property |
	property := properties at: identifier ifAbsent: [^self errorSymbolNotFound].
	^property at: key put: value! !

!LispTable methodsFor: 'removing' stamp: ''!
remove: symbol
	^properties removeKey: symbol ifAbsent: [^nil]! !

!LispTable methodsFor: 'property access' stamp: ''!
remprop: identifier key: key 
	| property |
	property := properties at: identifier ifAbsent: [^self errorSymbolNotFound].
	^property removeKey: key ifAbsent: [^nil]! !

ListController subclass: #PluggableYAListController
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Lisp-Pluggable-Support'!

!PluggableYAListController commentStamp: 'nishis 6/28/1998 09:39' prior: 0!
from Squeak 1.31!

!PluggableYAListController methodsFor: 'all' stamp: ''!
changeModelSelection: anInteger
	"Let the view handle this."

	view changeModelSelection: anInteger.! !

!PluggableYAListController methodsFor: 'all' stamp: ''!
controlActivity
	"Overridden to handle keystrokes."

	sensor keyboardPressed ifTrue: [view handleKeystroke: sensor keyboard].
	super controlActivity.! !

!PluggableYAListController methodsFor: 'all' stamp: ''!
yellowButtonActivity
	"Invoke the model's menu."

	| menu |
	menu _ view getMenu.
	menu == nil ifFalse: [menu invokeOn: model].! !

Array variableSubclass: #LispScannerTable
	instanceVariableNames: ''
	classVariableNames: 'ScanningTable'
	poolDictionaries: ''
	category: 'Lisp-Interpreter'!

!LispScannerTable class methodsFor: 'copyright' stamp: ''!
copyright
	^'Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.'! !

!LispScannerTable class methodsFor: 'table access' stamp: ''!
scannerTable
	| newTable |
	ScanningTable isNil
		ifTrue: 
			[newTable := self new: 256 withAll: #xBinary.
			newTable atAll: #(9 10 11 12 13 32) put: #xDelimiter.
			newTable atAll: ($0 asInteger to: $9 asInteger)
				put: #xDigit.
			newTable atAll: ($A asInteger to: $Z asInteger)
				put: #xSymbol.
			newTable atAll: ($a asInteger to: $z asInteger)
				put: #xSymbol.
			128 to: 256 do: [:i | newTable at: i put: #xSymbol].
			newTable at: $' asInteger put: #quote.
			newTable at: $" asInteger put: #xDoubleQuote.
			newTable at: ${ asInteger put: #xBrace.
			newTable at: $+ asInteger put: #xSign.
			newTable at: $- asInteger put: #xSign.
			newTable at: $< asInteger put: #xSymbol.
			newTable at: $> asInteger put: #xSymbol.
			newTable at: $= asInteger put: #xSymbol.
			newTable at: $~ asInteger put: #xSymbol.
			newTable at: $* asInteger put: #xSymbol.
			newTable at: $/ asInteger put: #xSymbol.
			newTable at: $_ asInteger put: #xSymbol.
			newTable at: $: asInteger put: #xSymbol.
			newTable at: $, asInteger put: #xSymbol.
			newTable at: $\ asInteger put: #xSymbol.
			newTable at: $% asInteger put: #xComment.
			newTable at: $( asInteger put: #leftParenthesis.
			newTable at: $) asInteger put: #rightParenthesis.
			newTable at: $[ asInteger put: #leftParenthesis.
			newTable at: $] asInteger put: #rightParenthesis.
			newTable at: $. asInteger put: #period.
			newTable at: $` asInteger put: #quote.
			newTable at: $!! asInteger put: #eof.
			ScanningTable := newTable].
	^ScanningTable! !

!LispScannerTable class methodsFor: 'copyright' stamp: ''!
system
	^'Goodies'! !

!LispScannerTable class methodsFor: 'copyright' stamp: ''!
version
	^'003'! !

!LispScannerTable methodsFor: 'accessing' stamp: ''!
at: index 
	(index isInteger and: [index between: 257 and: 65536])
		ifTrue: [(index - 1) asCharacter isLetter
				ifTrue: [^#xSymbol]
				ifFalse: [^#xBinary]]
		ifFalse: [^super at: index]! !

Array variableSubclass: #LispSmallTypeTable
	instanceVariableNames: 'value0 defaultValue letterValue digitValue separatorValue'
	classVariableNames: 'TypeTable'
	poolDictionaries: ''
	category: 'Lisp-Interpreter'!

!LispSmallTypeTable class methodsFor: 'copyright' stamp: ''!
copyright
	^'Copyright (C) 1995-1998 AOKI Atsushi, All Rights Reserved.'! !

!LispSmallTypeTable class methodsFor: 'copyright' stamp: ''!
system
	^'Goodies'! !

!LispSmallTypeTable class methodsFor: 'table access' stamp: ''!
typeTable
	| newTable |
	TypeTable isNil
		ifTrue: 
			[newTable := self new: 255 withAll: #xDefault.
			newTable atAllSeparatorsPut: #xDelimiter.
			newTable atAllDigitsPut: #xDigit.
			newTable atAllLettersPut: #xLetter.
			newTable at: $_ asInteger put: #xLetter.
			'!!%&*+,-/<=>?@\~' do: [:bin | newTable at: bin asInteger put: #xBinary].
			#(177 215 183 247) do: [:bin | newTable at: bin put: #xBinary].
			newTable at: $" asInteger put: #xDoubleQuote.
			newTable at: $$ asInteger put: #xDollar.
			newTable at: $' asInteger put: #xSingleQuote.
			newTable at: $# asInteger put: #literalQuote.
			newTable at: $( asInteger put: #leftParenthesis.
			newTable at: $) asInteger put: #rightParenthesis.
			newTable at: $. asInteger put: #period.
			newTable at: $: asInteger put: #colon.
			newTable at: $; asInteger put: #semicolon.
			newTable at: $[ asInteger put: #leftBracket.
			newTable at: $] asInteger put: #rightBracket.
			newTable at: $^ asInteger put: #upArrow.
			newTable at: $| asInteger put: #verticalBar.
			TypeTable := newTable].
	^TypeTable! !

!LispSmallTypeTable class methodsFor: 'copyright' stamp: ''!
version
	^'003'! !

!LispSmallTypeTable methodsFor: 'accessing' stamp: 'nishis 6/28/1998 09:38'!
at: index put: value 
	<primitive: 61>
	index isInteger ifFalse: [^super at: index put: value].
	(index >= 1 and: [index <= self basicSize])
		ifTrue: [^self errorImproperStore].
	index = 0 ifTrue: [^value0 := value].
	value = (self at: index) ifTrue: [^value].
	^self errorImproperStore! !

!LispSmallTypeTable methodsFor: 'accessing' stamp: ''!
atAllDigitsPut: value 
	digitValue := value.
	0 to: self size do: [:index | (Character value: index) isDigit ifTrue: [self at: index put: value]]! !

!LispSmallTypeTable methodsFor: 'accessing' stamp: ''!
atAllLettersPut: value 
	letterValue := value.
	0 to: self size do: [:index | (Character value: index) isLetter ifTrue: [self at: index put: value]]! !

!LispSmallTypeTable methodsFor: 'accessing' stamp: ''!
atAllPut: value 
	value0 := value.
	defaultValue := value.
	letterValue := value.
	digitValue := value.
	separatorValue := value.
	^super atAllPut: value! !

!LispSmallTypeTable methodsFor: 'accessing' stamp: ''!
atAllSeparatorsPut: value 
	separatorValue := value.
	0 to: self size do: [:index | (Character value: index) isSeparator ifTrue: [self at: index put: value]]! !

!LispSmallTypeTable methodsFor: 'accessing' stamp: ''!
atDefaultPut: value 
	defaultValue := value! !

!LispSmallTypeTable methodsFor: 'accessing' stamp: 'nishis 6/28/1998 07:03'!
basicAt: index put: value 
	<primitive: 61>
	index isInteger ifFalse: [^super basicAt: index put: value].
	(index >= 1 and: [index <= self basicSize])
		ifTrue: [^self errorImproperStore].
	^super errorSubscriptBounds: index! !

!LispSmallTypeTable methodsFor: 'error handling' stamp: 'nishis 6/28/1998 06:50'!
subscriptBoundsError: index 
	| char |
	((index between: self size + 1 and: Character characterTable size)
		or: [index = 0])
		ifFalse: [^super errorSubscriptBounds: index].
	index = 0 ifTrue: [^value0].
	char := Character value: index.
	char isLetter ifTrue: [^letterValue].
	char isSeparator ifTrue: [^separatorValue].
	char isDigit ifTrue: [^digitValue].
	^defaultValue! !
LispCons initialize!
LispNil initialize!
